@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -125,7 +125,9 @@ bool isPathAbsolute(const QString &path)
 }
 
 // If the type does not already exist as a file import, add the type and return the new type
-QQmlType *getTypeForUrl(const QString &urlString, const QHashedStringRef& typeName, bool isCompositeSingleton, QList<QQmlError> *errors)
+QQmlType *getTypeForUrl(const QString &urlString, const QHashedStringRef& typeName,
+                        bool isCompositeSingleton, QList<QQmlError> *errors,
+                        int majorVersion=-1, int minorVersion=-1)
 {
     QUrl url(urlString);
     QQmlType *ret = QQmlMetaType::qmlType(url);
@@ -140,8 +142,8 @@ QQmlType *getTypeForUrl(const QString &urlString, const QHashedStringRef& typeNa
             QQmlPrivate::RegisterCompositeSingletonType reg = {
                 url,
                 "", //Empty URI indicates loaded via file imports
-                -1,
-                -1,
+                majorVersion,
+                minorVersion,
                 buf.constData()
             };
             ret = QQmlMetaType::qmlTypeFromIndex(QQmlPrivate::qmlregister(QQmlPrivate::CompositeSingletonRegistration, &reg));
@@ -149,8 +151,8 @@ QQmlType *getTypeForUrl(const QString &urlString, const QHashedStringRef& typeNa
             QQmlPrivate::RegisterCompositeType reg = {
                 url,
                 "", //Empty URI indicates loaded via file imports
-                -1,
-                -1,
+                majorVersion,
+                minorVersion,
                 buf.constData()
             };
             ret = QQmlMetaType::qmlTypeFromIndex(QQmlPrivate::qmlregister(QQmlPrivate::CompositeRegistration, &reg));
@@ -169,6 +171,7 @@ QQmlType *getTypeForUrl(const QString &urlString, const QHashedStringRef& typeNa
 
 } // namespace
 
+#ifndef QT_NO_LIBRARY
 struct RegisteredPlugin {
     QString uri;
     QPluginLoader* loader;
@@ -193,6 +196,7 @@ void qmlClearEnginePlugins()
 }
 
 typedef QPair<QStaticPlugin, QJsonArray> StaticPluginPair;
+#endif
 
 class QQmlImportNamespace
 {
@@ -292,9 +296,10 @@ public:
                                                       const QString &uri, const QString &url,
                                                       int vmaj, int vmin, QV4::CompiledData::Import::ImportType type,
                                                       QList<QQmlError> *errors, bool lowPrecedence = false);
-
+#ifndef QT_NO_LIBRARY
    bool populatePluginPairVector(QVector<StaticPluginPair> &result, const QString &uri,
                                      const QString &qmldirPath, QList<QQmlError> *errors);
+#endif
 };
 
 /*!
@@ -382,7 +387,7 @@ void QQmlImports::populateCache(QQmlTypeNameCache *cache) const
 // We need to exclude the entry for the current baseUrl. This can happen for example
 // when handling qmldir files on the remote dir case and the current type is marked as
 // singleton.
-bool excludeBaseUrl(const QString &importUrl, const QString &fileName, const QString baseUrl)
+bool excludeBaseUrl(const QString &importUrl, const QString &fileName, const QString &baseUrl)
 {
     if (importUrl.isEmpty())
         return false;
@@ -398,7 +403,7 @@ bool excludeBaseUrl(const QString &importUrl, const QString &fileName, const QSt
     return true;
 }
 
-void findCompositeSingletons(const QQmlImportNamespace &set, QList<QQmlImports::CompositeSingletonReference> &resultList, QUrl baseUrl)
+void findCompositeSingletons(const QQmlImportNamespace &set, QList<QQmlImports::CompositeSingletonReference> &resultList, const QUrl &baseUrl)
 {
     typedef QQmlDirComponents::const_iterator ConstIterator;
 
@@ -413,6 +418,8 @@ void findCompositeSingletons(const QQmlImportNamespace &set, QList<QQmlImports::
                 QQmlImports::CompositeSingletonReference ref;
                 ref.typeName = cit->typeName;
                 ref.prefix = set.prefix;
+                ref.majorVersion = cit->majorVersion;
+                ref.minorVersion = cit->minorVersion;
                 resultList.append(ref);
             }
         }
@@ -653,9 +660,13 @@ bool QQmlImportNamespace::Import::resolveType(QQmlTypeLoader *typeLoader,
         }
 
         if (candidate != end) {
+            int major = vmajor ? *vmajor : -1;
+            int minor = vminor ? *vminor : -1;
+            QQmlType *returnType = getTypeForUrl(componentUrl, type, isCompositeSingleton, 0,
+                                                 major, minor);
             if (type_return)
-                *type_return = getTypeForUrl(componentUrl, type, isCompositeSingleton, 0);
-            return (*type_return != 0);
+                *type_return = returnType;
+            return returnType != 0;
         }
     } else if (!isLibrary) {
         QString qmlUrl;
@@ -667,15 +678,7 @@ bool QQmlImportNamespace::Import::resolveType(QQmlTypeLoader *typeLoader,
         };
         for (uint i = 0; i < sizeof(urlsToTry) / sizeof(urlsToTry[0]); ++i) {
             const QString url = urlsToTry[i];
-
-            if (QQmlFile::isBundle(url)) {
-                exists = QQmlFile::bundleFileExists(url, typeLoader->engine());
-            } else {
-                exists = !typeLoader->absoluteFilePath(QQmlFile::urlToLocalFileOrQrc(url)).isEmpty();
-                if (!exists)
-                    exists = QQmlMetaType::findCachedCompilationUnit(QUrl(url));
-            }
-
+            exists = !typeLoader->absoluteFilePath(QQmlFile::urlToLocalFileOrQrc(url)).isEmpty();
             if (exists) {
                 qmlUrl = url;
                 break;
@@ -687,9 +690,10 @@ bool QQmlImportNamespace::Import::resolveType(QQmlTypeLoader *typeLoader,
                 if (typeRecursionDetected)
                     *typeRecursionDetected = true;
             } else {
+                QQmlType *returnType = getTypeForUrl(qmlUrl, type, false, 0);
                 if (type_return)
-                    *type_return = getTypeForUrl(qmlUrl, type, false, 0);
-                return (*type_return) != 0;
+                    *type_return = returnType;
+                return returnType != 0;
             }
         }
     }
@@ -832,6 +836,7 @@ QQmlImportNamespace *QQmlImportsPrivate::findQualifiedNamespace(const QHashedStr
 }
 
 
+#ifndef QT_NO_LIBRARY
 /*!
     Get all static plugins that are QML plugins and has a meta data URI that begins with \a uri.
     Note that if e.g uri == "a", and different plugins have meta data "a", "a.2.1", "a.b.c", all
@@ -846,7 +851,7 @@ bool QQmlImportsPrivate::populatePluginPairVector(QVector<StaticPluginPair> &res
         // the list the first time called to only contain QML plugins:
         foreach (const QStaticPlugin &plugin, QPluginLoader::staticPlugins()) {
             if (qobject_cast<QQmlExtensionPlugin *>(plugin.instance()))
-               plugins.append(plugin);
+                plugins.append(plugin);
         }
     }
 
@@ -875,11 +880,12 @@ bool QQmlImportsPrivate::populatePluginPairVector(QVector<StaticPluginPair> &res
     }
     return true;
 }
+#endif
 
 /*!
 Import an extension defined by a qmldir file.
 
-\a qmldirFilePath is either a raw file path, or a bundle url.
+\a qmldirFilePath is a raw file path.
 */
 bool QQmlImportsPrivate::importExtension(const QString &qmldirFilePath,
                                          const QString &uri,
@@ -930,7 +936,7 @@ bool QQmlImportsPrivate::importExtension(const QString &qmldirFilePath,
             QString resolvedFilePath = database->resolvePlugin(typeLoader, qmldirPath, plugin.path, plugin.name);
             if (!resolvedFilePath.isEmpty()) {
                 dynamicPluginsFound++;
-                if (!database->importDynamicPlugin(resolvedFilePath, uri, typeNamespace, errors)) {
+                if (!database->importDynamicPlugin(resolvedFilePath, uri, typeNamespace, vmaj, errors)) {
                     if (errors) {
                         // XXX TODO: should we leave the import plugin error alone?
                         // Here, we pop it off the top and coalesce it into this error's message.
@@ -966,7 +972,7 @@ bool QQmlImportsPrivate::importExtension(const QString &qmldirFilePath,
                         if (versionUri == metaTagUri.toString()) {
                             staticPluginsFound++;
                             QObject *instance = pair.first.instance();
-                            if (!database->importStaticPlugin(instance, basePath, uri, typeNamespace, errors)) {
+                            if (!database->importStaticPlugin(instance, basePath, uri, typeNamespace, vmaj, errors)) {
                                 if (errors) {
                                     QQmlError poppedError = errors->takeFirst();
                                     QQmlError error;
@@ -1001,6 +1007,13 @@ bool QQmlImportsPrivate::importExtension(const QString &qmldirFilePath,
     }
 
 #else
+    Q_UNUSED(qmldirFilePath);
+    Q_UNUSED(uri);
+    Q_UNUSED(vmaj);
+    Q_UNUSED(vmin);
+    Q_UNUSED(database);
+    Q_UNUSED(qmldir);
+    Q_UNUSED(errors);
     return false;
 #endif // QT_NO_LIBRARY
     return true;
@@ -1012,17 +1025,11 @@ bool QQmlImportsPrivate::getQmldirContent(const QString &qmldirIdentifier, const
     Q_ASSERT(errors);
     Q_ASSERT(qmldir);
 
-    *qmldir = typeLoader->qmldirContent(qmldirIdentifier, uri);
+    *qmldir = typeLoader->qmldirContent(qmldirIdentifier);
     if (*qmldir) {
         // Ensure that parsing was successful
         if ((*qmldir)->hasError()) {
-            QUrl url;
-
-            if (QQmlFile::isBundle(qmldirIdentifier))
-                url = QUrl(qmldirIdentifier);
-            else
-                url = QUrl::fromLocalFile(qmldirIdentifier);
-
+            QUrl url = QUrl::fromLocalFile(qmldirIdentifier);
             const QList<QQmlError> qmldirErrors = (*qmldir)->errors(uri);
             for (int i = 0; i < qmldirErrors.size(); ++i) {
                 QQmlError error = qmldirErrors.at(i);
@@ -1328,30 +1335,7 @@ bool QQmlImportsPrivate::addFileImport(const QString& uri, const QString &prefix
 
     QString qmldirIdentifier;
 
-    if (QQmlFile::isBundle(qmldirUrl)) {
-
-        QString dir = resolveLocalUrl(base, importUri);
-        Q_ASSERT(QQmlFile::isBundle(dir));
-        if (!QQmlFile::bundleDirectoryExists(dir, typeLoader->engine())) {
-            if (!isImplicitImport) {
-                QQmlError error;
-                error.setDescription(QQmlImportDatabase::tr("\"%1\": no such directory").arg(uri));
-                error.setUrl(QUrl(qmldirUrl));
-                errors->prepend(error);
-            }
-            return false;
-        }
-
-        // Transforms the (possible relative) uri into our best guess relative to the
-        // import paths.
-        importUri = resolvedUri(dir, database);
-        if (importUri.endsWith(Slash))
-            importUri.chop(1);
-
-        if (QQmlFile::bundleFileExists(qmldirUrl, typeLoader->engine()))
-            qmldirIdentifier = qmldirUrl;
-
-    } else if (QQmlFile::isLocalFile(qmldirUrl)) {
+    if (QQmlFile::isLocalFile(qmldirUrl)) {
 
         QString localFileOrQrc = QQmlFile::urlToLocalFileOrQrc(qmldirUrl);
         Q_ASSERT(!localFileOrQrc.isEmpty());
@@ -1555,12 +1539,12 @@ bool QQmlImports::locateQmldir(QQmlImportDatabase *importDb,
 
 bool QQmlImports::isLocal(const QString &url)
 {
-    return QQmlFile::isBundle(url) || !QQmlFile::urlToLocalFileOrQrc(url).isEmpty();
+    return !QQmlFile::urlToLocalFileOrQrc(url).isEmpty();
 }
 
 bool QQmlImports::isLocal(const QUrl &url)
 {
-    return QQmlFile::isBundle(url) || !QQmlFile::urlToLocalFileOrQrc(url).isEmpty();
+    return !QQmlFile::urlToLocalFileOrQrc(url).isEmpty();
 }
 
 void QQmlImports::setDesignerSupportRequired(bool b)
@@ -1585,8 +1569,8 @@ QQmlImportDatabase::QQmlImportDatabase(QQmlEngine *e)
     addImportPath(installImportsPath);
 
     // env import paths
-    QByteArray envImportPath = qgetenv("QML2_IMPORT_PATH");
-    if (!envImportPath.isEmpty()) {
+    if (Q_UNLIKELY(!qEnvironmentVariableIsEmpty("QML2_IMPORT_PATH"))) {
+        const QByteArray envImportPath = qgetenv("QML2_IMPORT_PATH");
 #if defined(Q_OS_WIN)
         QLatin1Char pathSep(';');
 #else
@@ -1674,9 +1658,7 @@ QString QQmlImportDatabase::resolvePlugin(QQmlTypeLoader *typeLoader,
   \header \li Platform \li Valid suffixes
   \row \li Windows     \li \c .dll
   \row \li Unix/Linux  \li \c .so
-  \row \li AIX  \li \c .a
-  \row \li HP-UX       \li \c .sl, \c .so (HP-UXi)
-  \row \li Mac OS X    \li \c .dylib, \c .bundle, \c .so
+  \row \li \macos    \li \c .dylib, \c .bundle, \c .so
   \endtable
 
   Version number on unix are ignored.
@@ -1692,9 +1674,7 @@ QString QQmlImportDatabase::resolvePlugin(QQmlTypeLoader *typeLoader,
                          << QLatin1String("d.dll") // try a qmake-style debug build first
 # endif
                          << QLatin1String(".dll"));
-#else
-
-# if defined(Q_OS_DARWIN)
+#elif defined(Q_OS_DARWIN)
 
     return resolvePlugin(typeLoader, qmldirPath, qmldirPluginPath, baseName,
                          QStringList()
@@ -1708,31 +1688,8 @@ QString QQmlImportDatabase::resolvePlugin(QQmlTypeLoader *typeLoader,
                          << QLatin1String(".so")
                          << QLatin1String(".bundle"),
                          QLatin1String("lib"));
-# else  // Generic Unix
-    QStringList validSuffixList;
-
-#  if defined(Q_OS_HPUX)
-/*
-    See "HP-UX Linker and Libraries User's Guide", section "Link-time Differences between PA-RISC and IPF":
-    "In PA-RISC (PA-32 and PA-64) shared libraries are suffixed with .sl. In IPF (32-bit and 64-bit),
-    the shared libraries are suffixed with .so. For compatibility, the IPF linker also supports the .sl suffix."
- */
-    validSuffixList << QLatin1String(".sl");
-#   if defined __ia64
-    validSuffixList << QLatin1String(".so");
-#   endif
-#  elif defined(Q_OS_AIX)
-    validSuffixList << QLatin1String(".a") << QLatin1String(".so");
-#  elif defined(Q_OS_UNIX)
-    validSuffixList << QLatin1String(".so");
-#  endif
-
-    // Examples of valid library names:
-    //  libfoo.so
-
-    return resolvePlugin(typeLoader, qmldirPath, qmldirPluginPath, baseName, validSuffixList, QLatin1String("lib"));
-# endif
-
+# else  // Unix
+    return resolvePlugin(typeLoader, qmldirPath, qmldirPluginPath, baseName, QStringList() << QLatin1String(".so"), QLatin1String("lib"));
 #endif
 }
 
@@ -1844,7 +1801,7 @@ void QQmlImportDatabase::setImportPathList(const QStringList &paths)
     \internal
 */
 bool QQmlImportDatabase::registerPluginTypes(QObject *instance, const QString &basePath,
-                                      const QString &uri, const QString &typeNamespace, QList<QQmlError> *errors)
+                                      const QString &uri, const QString &typeNamespace, int vmaj, QList<QQmlError> *errors)
 {
     if (qmlImportTrace())
         qDebug().nospace() << "QQmlImportDatabase::registerPluginTypes: " << uri << " from " << basePath;
@@ -1866,7 +1823,7 @@ bool QQmlImportDatabase::registerPluginTypes(QObject *instance, const QString &b
     {
         // Create a scope for QWriteLocker to keep it as narrow as possible, and
         // to ensure that we release it before the call to initalizeEngine below
-        QWriteLocker lock(QQmlMetaType::typeRegistrationLock());
+        QMutexLocker lock(QQmlMetaType::typeRegistrationLock());
 
         if (!typeNamespace.isEmpty()) {
             // This is an 'identified' module
@@ -1880,7 +1837,7 @@ bool QQmlImportDatabase::registerPluginTypes(QObject *instance, const QString &b
                 return false;
             }
 
-            if (QQmlMetaType::namespaceContainsRegistrations(typeNamespace)) {
+            if (QQmlMetaType::namespaceContainsRegistrations(typeNamespace, vmaj)) {
                 // Other modules have already installed to this namespace
                 if (errors) {
                     QQmlError error;
@@ -1927,7 +1884,7 @@ bool QQmlImportDatabase::registerPluginTypes(QObject *instance, const QString &b
     \internal
 */
 bool QQmlImportDatabase::importStaticPlugin(QObject *instance, const QString &basePath,
-                                      const QString &uri, const QString &typeNamespace, QList<QQmlError> *errors)
+                                      const QString &uri, const QString &typeNamespace, int vmaj, QList<QQmlError> *errors)
 {
 #ifndef QT_NO_LIBRARY
     // Dynamic plugins are differentiated by their filepath. For static plugins we
@@ -1951,7 +1908,7 @@ bool QQmlImportDatabase::importStaticPlugin(QObject *instance, const QString &ba
         plugin.loader = 0;
         plugins->insert(uniquePluginID, plugin);
 
-        if (!registerPluginTypes(instance, basePath, uri, typeNamespace, errors))
+        if (!registerPluginTypes(instance, basePath, uri, typeNamespace, vmaj, errors))
             return false;
     }
 
@@ -1966,6 +1923,12 @@ bool QQmlImportDatabase::importStaticPlugin(QObject *instance, const QString &ba
 
     return true;
 #else
+    Q_UNUSED(instance);
+    Q_UNUSED(basePath);
+    Q_UNUSED(uri);
+    Q_UNUSED(typeNamespace);
+    Q_UNUSED(vmaj);
+    Q_UNUSED(errors);
     return false;
 #endif
 }
@@ -1974,7 +1937,7 @@ bool QQmlImportDatabase::importStaticPlugin(QObject *instance, const QString &ba
     \internal
 */
 bool QQmlImportDatabase::importDynamicPlugin(const QString &filePath, const QString &uri,
-                                             const QString &typeNamespace, QList<QQmlError> *errors)
+                                             const QString &typeNamespace, int vmaj, QList<QQmlError> *errors)
 {
 #ifndef QT_NO_LIBRARY
     QFileInfo fileInfo(filePath);
@@ -2010,9 +1973,8 @@ bool QQmlImportDatabase::importDynamicPlugin(const QString &filePath, const QStr
                     QQmlError error;
                     error.setDescription(loader->errorString());
                     errors->prepend(error);
-
-                    delete loader;
                 }
+                delete loader;
                 return false;
             }
         } else {
@@ -2028,7 +1990,7 @@ bool QQmlImportDatabase::importDynamicPlugin(const QString &filePath, const QStr
             plugins->insert(absoluteFilePath, plugin);
 
             // Continue with shared code path for dynamic and static plugins:
-            if (!registerPluginTypes(instance, fileInfo.absolutePath(), uri, typeNamespace, errors))
+            if (!registerPluginTypes(instance, fileInfo.absolutePath(), uri, typeNamespace, vmaj, errors))
                 return false;
         }
 
@@ -2047,6 +2009,11 @@ bool QQmlImportDatabase::importDynamicPlugin(const QString &filePath, const QStr
 
     return true;
 #else
+    Q_UNUSED(filePath);
+    Q_UNUSED(uri);
+    Q_UNUSED(typeNamespace);
+    Q_UNUSED(vmaj);
+    Q_UNUSED(errors);
     return false;
 #endif
 }
