@@ -39,6 +39,10 @@
 
 QT_BEGIN_NAMESPACE
 
+namespace QTest {
+    extern int Q_TESTLIB_EXPORT defaultMouseDelay();
+}
+
 QuickTestEvent::QuickTestEvent(QObject *parent)
     : QObject(parent)
 {
@@ -48,6 +52,11 @@ QuickTestEvent::~QuickTestEvent()
 {
 }
 
+int QuickTestEvent::defaultMouseDelay() const
+{
+    return QTest::defaultMouseDelay();
+}
+
 bool QuickTestEvent::keyPress(int key, int modifiers, int delay)
 {
     QWindow *window = activeWindow();
@@ -105,10 +114,6 @@ bool QuickTestEvent::keyClickChar(const QString &character, int modifiers, int d
     return true;
 }
 
-namespace QTest {
-    extern int Q_TESTLIB_EXPORT defaultMouseDelay();
-};
-
 namespace QtQuickTest
 {
     enum MouseAction { MousePress, MouseRelease, MouseClick, MouseDoubleClick, MouseMove, MouseDoubleClickSequence };
