@@ -211,8 +211,8 @@ public:
     bool interactive : 1;
     bool calcVelocity : 1;
     bool pixelAligned : 1;
-    bool replayingPressEvent : 1;
     QElapsedTimer timer;
+    QBasicTimer movementEndingTimer;
     qint64 lastPosTime;
     qint64 lastPressTime;
     QPointF lastPos;
