@@ -1,72 +1,75 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include <private/qquickshadereffectnode_p.h>
 
-#include "qquickshadereffectmesh_p.h"
 #include "qquickshadereffect_p.h"
 #include <QtQuick/qsgtextureprovider.h>
 #include <QtQuick/private/qsgrenderer_p.h>
 #include <QtQuick/private/qsgshadersourcebuilder_p.h>
+#include <QtQuick/private/qsgtexture_p.h>
+#include <QtCore/qmutex.h>
 
 QT_BEGIN_NAMESPACE
 
+static bool hasAtlasTexture(const QVector<QSGTextureProvider *> &textureProviders)
+{
+    for (int i = 0; i < textureProviders.size(); ++i) {
+        QSGTextureProvider *t = textureProviders.at(i);
+        if (t && t->texture() && t->texture()->isAtlasTexture())
+            return true;
+    }
+    return false;
+}
+
 class QQuickCustomMaterialShader : public QSGMaterialShader
 {
 public:
     QQuickCustomMaterialShader(const QQuickShaderEffectMaterialKey &key, const QVector<QByteArray> &attributes);
-    virtual void deactivate();
-    virtual void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect);
-    virtual char const *const *attributeNames() const;
+    void deactivate() Q_DECL_OVERRIDE;
+    void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect) Q_DECL_OVERRIDE;
+    char const *const *attributeNames() const Q_DECL_OVERRIDE;
 
 protected:
     friend class QQuickShaderEffectNode;
 
-    virtual void compile();
-    virtual const char *vertexShader() const;
-    virtual const char *fragmentShader() const;
+    void compile() Q_DECL_OVERRIDE;
+    const char *vertexShader() const Q_DECL_OVERRIDE;
+    const char *fragmentShader() const Q_DECL_OVERRIDE;
 
     const QQuickShaderEffectMaterialKey m_key;
+    QVector<QByteArray> m_attributes;
     QVector<const char *> m_attributeNames;
-    const QVector<QByteArray> m_attributes;
     QString m_log;
     bool m_compiled;
 
@@ -80,15 +83,17 @@ QQuickCustomMaterialShader::QQuickCustomMaterialShader(const QQuickShaderEffectM
     , m_compiled(false)
     , m_initialized(false)
 {
-    for (int i = 0; i < attributes.count(); ++i)
-        m_attributeNames.append(attributes.at(i).constData());
+    const int attributesCount = m_attributes.count();
+    m_attributeNames.reserve(attributesCount + 1);
+    for (int i = 0; i < attributesCount; ++i)
+        m_attributeNames.append(m_attributes.at(i).constData());
     m_attributeNames.append(0);
 }
 
 void QQuickCustomMaterialShader::deactivate()
 {
     QSGMaterialShader::deactivate();
-    glDisable(GL_CULL_FACE);
+    QOpenGLContext::currentContext()->functions()->glDisable(GL_CULL_FACE);
 }
 
 void QQuickCustomMaterialShader::updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *oldEffect)
@@ -136,17 +141,23 @@ void QQuickCustomMaterialShader::updateState(const RenderState &state, QSGMateri
                 functions->glActiveTexture(GL_TEXTURE0 + idx);
                 if (QSGTextureProvider *provider = material->textureProviders.at(idx)) {
                     if (QSGTexture *texture = provider->texture()) {
+
+#ifndef QT_NO_DEBUG
+                        if (!qsg_safeguard_texture(texture))
+                            continue;
+#endif
+
                         if (loc >= 0) {
                             QRectF r = texture->normalizedTextureSubRect();
                             program()->setUniformValue(loc, r.x(), r.y(), r.width(), r.height());
-                        } else if (texture->isAtlasTexture()) {
+                        } else if (texture->isAtlasTexture() && !material->geometryUsesTextureSubRect) {
                             texture = texture->removedFromAtlas();
                         }
                         texture->bind();
                         continue;
                     }
                 }
-                glBindTexture(GL_TEXTURE_2D, 0);
+                functions->glBindTexture(GL_TEXTURE_2D, 0);
             } else if (d.specialType == UniformData::Opacity) {
                 program()->setUniformValue(loc, state.opacity());
             } else if (d.specialType == UniformData::Matrix) {
@@ -217,15 +228,15 @@ void QQuickCustomMaterialShader::updateState(const RenderState &state, QSGMateri
     if (oldEffect == 0 || material->cullMode != oldMaterial->cullMode) {
         switch (material->cullMode) {
         case QQuickShaderEffectMaterial::FrontFaceCulling:
-            glEnable(GL_CULL_FACE);
-            glCullFace(GL_FRONT);
+            functions->glEnable(GL_CULL_FACE);
+            functions->glCullFace(GL_FRONT);
             break;
         case QQuickShaderEffectMaterial::BackFaceCulling:
-            glEnable(GL_CULL_FACE);
-            glCullFace(GL_BACK);
+            functions->glEnable(GL_CULL_FACE);
+            functions->glCullFace(GL_BACK);
             break;
         default:
-            glDisable(GL_CULL_FACE);
+            functions->glDisable(GL_CULL_FACE);
             break;
         }
     }
@@ -256,7 +267,7 @@ void QQuickCustomMaterialShader::compile()
     char const *const *attr = attributeNames();
 #ifndef QT_NO_DEBUG
     int maxVertexAttribs = 0;
-    glGetIntegerv(GL_MAX_VERTEX_ATTRIBS, &maxVertexAttribs);
+    QOpenGLContext::currentContext()->functions()->glGetIntegerv(GL_MAX_VERTEX_ATTRIBS, &maxVertexAttribs);
     int attrCount = 0;
     while (attrCount < maxVertexAttribs && attr[attrCount])
         ++attrCount;
@@ -288,8 +299,8 @@ void QQuickCustomMaterialShader::compile()
 
         QSGShaderSourceBuilder::initializeProgramFromFiles(
             program(),
-            QStringLiteral(":/items/shaders/shadereffectfallback.vert"),
-            QStringLiteral(":/items/shaders/shadereffectfallback.frag"));
+            QStringLiteral(":/qt-project.org/items/shaders/shadereffectfallback.vert"),
+            QStringLiteral(":/qt-project.org/items/shaders/shadereffectfallback.frag"));
 
 #ifndef QT_NO_DEBUG
         for (int i = 0; i < attrCount; ++i) {
@@ -316,8 +327,6 @@ const char *QQuickCustomMaterialShader::fragmentShader() const
 
 bool QQuickShaderEffectMaterialKey::operator == (const QQuickShaderEffectMaterialKey &other) const
 {
-    if (className != other.className)
-        return false;
     for (int shaderType = 0; shaderType < ShaderTypeCount; ++shaderType) {
         if (sourceCode[shaderType] != other.sourceCode[shaderType])
             return false;
@@ -325,20 +334,40 @@ bool QQuickShaderEffectMaterialKey::operator == (const QQuickShaderEffectMateria
     return true;
 }
 
+bool QQuickShaderEffectMaterialKey::operator != (const QQuickShaderEffectMaterialKey &other) const
+{
+    return !(*this == other);
+}
+
 uint qHash(const QQuickShaderEffectMaterialKey &key)
 {
-    uint hash = qHash((void *)key.className);
+    uint hash = 1;
     typedef QQuickShaderEffectMaterialKey Key;
     for (int shaderType = 0; shaderType < Key::ShaderTypeCount; ++shaderType)
         hash = hash * 31337 + qHash(key.sourceCode[shaderType]);
     return hash;
 }
 
-
-QHash<QQuickShaderEffectMaterialKey, QSharedPointer<QSGMaterialType> > QQuickShaderEffectMaterial::materialMap;
+class QQuickShaderEffectMaterialCache : public QObject
+{
+    Q_OBJECT
+public:
+    static QQuickShaderEffectMaterialCache *get(bool create = true) {
+        QOpenGLContext *ctx = QOpenGLContext::currentContext();
+        QQuickShaderEffectMaterialCache *me = ctx->findChild<QQuickShaderEffectMaterialCache *>(QStringLiteral("__qt_ShaderEffectCache"), Qt::FindDirectChildrenOnly);
+        if (!me && create) {
+            me = new QQuickShaderEffectMaterialCache();
+            me->setObjectName(QStringLiteral("__qt_ShaderEffectCache"));
+            me->setParent(ctx);
+        }
+        return me;
+    }
+    QHash<QQuickShaderEffectMaterialKey, QSGMaterialType *> cache;
+};
 
 QQuickShaderEffectMaterial::QQuickShaderEffectMaterial(QQuickShaderEffectNode *node)
     : cullMode(NoCulling)
+    , geometryUsesTextureSubRect(false)
     , m_node(node)
     , m_emittedLogChanged(false)
 {
@@ -347,7 +376,7 @@ QQuickShaderEffectMaterial::QQuickShaderEffectMaterial(QQuickShaderEffectNode *n
 
 QSGMaterialType *QQuickShaderEffectMaterial::type() const
 {
-    return m_type.data();
+    return m_type;
 }
 
 QSGMaterialShader *QQuickShaderEffectMaterial::createShader() const
@@ -355,19 +384,73 @@ QSGMaterialShader *QQuickShaderEffectMaterial::createShader() const
     return new QQuickCustomMaterialShader(m_source, attributes);
 }
 
-int QQuickShaderEffectMaterial::compare(const QSGMaterial *other) const
+bool QQuickShaderEffectMaterial::UniformData::operator == (const UniformData &other) const
 {
-    return this - static_cast<const QQuickShaderEffectMaterial *>(other);
+    if (specialType != other.specialType)
+        return false;
+    if (name != other.name)
+        return false;
+
+    if (specialType == UniformData::Sampler) {
+        // We can't check the source objects as these live in the GUI thread,
+        // so return true here and rely on the textureProvider check for
+        // equality of these..
+        return true;
+    } else {
+        return value == other.value;
+    }
+}
+
+int QQuickShaderEffectMaterial::compare(const QSGMaterial *o) const
+{
+    const QQuickShaderEffectMaterial *other = static_cast<const QQuickShaderEffectMaterial *>(o);
+    if ((hasAtlasTexture(textureProviders) && !geometryUsesTextureSubRect) || (hasAtlasTexture(other->textureProviders) && !other->geometryUsesTextureSubRect))
+        return 1;
+    if (cullMode != other->cullMode)
+        return 1;
+    for (int shaderType = 0; shaderType < QQuickShaderEffectMaterialKey::ShaderTypeCount; ++shaderType) {
+        if (uniforms[shaderType] != other->uniforms[shaderType])
+            return 1;
+    }
+
+    // Check the texture providers..
+    if (textureProviders.size() != other->textureProviders.size())
+        return 1;
+    for (int i=0; i<textureProviders.size(); ++i) {
+        QSGTextureProvider *tp1 = textureProviders.at(i);
+        QSGTextureProvider *tp2 = other->textureProviders.at(i);
+        if (!tp1 || !tp2)
+            return tp1 == tp2 ? 0 : 1;
+        QSGTexture *t1 = tp1->texture();
+        QSGTexture *t2 = tp2->texture();
+        if (!t1 || !t2)
+            return t1 == t2 ? 0 : 1;
+        // Check texture id's as textures may be in the same atlas.
+        if (t1->textureId() != t2->textureId())
+            return 1;
+    }
+    return 0;
 }
 
 void QQuickShaderEffectMaterial::setProgramSource(const QQuickShaderEffectMaterialKey &source)
 {
     m_source = source;
     m_emittedLogChanged = false;
-    m_type = materialMap.value(m_source);
-    if (m_type.isNull()) {
-        m_type = QSharedPointer<QSGMaterialType>(new QSGMaterialType);
-        materialMap.insert(m_source, m_type);
+
+    QQuickShaderEffectMaterialCache *cache = QQuickShaderEffectMaterialCache::get();
+    m_type = cache->cache.value(m_source);
+    if (!m_type) {
+        m_type = new QSGMaterialType();
+        cache->cache.insert(source, m_type);
+    }
+}
+
+void QQuickShaderEffectMaterial::cleanupMaterialCache()
+{
+    QQuickShaderEffectMaterialCache *cache = QQuickShaderEffectMaterialCache::get(false);
+    if (cache) {
+        qDeleteAll(cache->cache.values());
+        delete cache;
     }
 }
 
@@ -406,6 +489,7 @@ QQuickShaderEffectNode::~QQuickShaderEffectNode()
 void QQuickShaderEffectNode::markDirtyTexture()
 {
     markDirty(DirtyMaterial);
+    Q_EMIT dirtyTexture();
 }
 
 void QQuickShaderEffectNode::textureProviderDestroyed(QObject *object)
@@ -420,4 +504,6 @@ void QQuickShaderEffectNode::preprocess()
     static_cast<QQuickShaderEffectMaterial *>(material())->updateTextures();
 }
 
+#include "qquickshadereffectnode.moc"
+
 QT_END_NAMESPACE
