@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,21 +34,21 @@
 #include "qquicktextedit_p.h"
 #include "qquicktextedit_p_p.h"
 #include "qquicktextcontrol_p.h"
-#include "qquicktext_p_p.h"
+#include "qquicktextdocument_p.h"
 #include "qquickevents_p_p.h"
 #include "qquickwindow.h"
 #include "qquicktextnode_p.h"
 #include "qquicktextnodeengine_p.h"
 #include "qquicktextutil_p.h"
-#include <QtQuick/qsgsimplerectnode.h>
 
-#include <QtQml/qqmlinfo.h>
+#include <QtCore/qmath.h>
 #include <QtGui/qguiapplication.h>
 #include <QtGui/qevent.h>
 #include <QtGui/qpainter.h>
 #include <QtGui/qtextobject.h>
 #include <QtGui/qtexttable.h>
-#include <QtCore/qmath.h>
+#include <QtQml/qqmlinfo.h>
+#include <QtQuick/qsgsimplerectnode.h>
 
 #include <private/qqmlglobal_p.h>
 #include <private/qqmlproperty_p.h>
@@ -112,15 +104,17 @@ TextEdit {
     You can translate between cursor positions (characters from the start of the document) and pixel
     points using positionAt() and positionToRectangle().
 
-    \sa Text, TextInput, {examples/quick/text/textselection}{Text Selection example}
+    \sa Text, TextInput
 */
 
 /*!
-    \qmlsignal QtQuick::TextEdit::onLinkActivated(string link)
+    \qmlsignal QtQuick::TextEdit::linkActivated(string link)
 
-    This handler is called when the user clicks on a link embedded in the text.
+    This signal is emitted when the user clicks on a link embedded in the text.
     The link must be in rich text or HTML format and the
     \a link string provides access to the particular link.
+
+    The corresponding handler is \c onLinkActivated.
 */
 
 // This is a pretty arbitrary figure. The idea is that we don't want to break down the document
@@ -153,17 +147,19 @@ namespace {
             newNode->setFlag(QSGNode::OwnedByParent);
         }
 
-        void resetCursorNode(QSGSimpleRectNode* newNode)
+        void resetCursorNode(QSGRectangleNode* newNode)
         {
             if (cursorNode)
                 removeChildNode(cursorNode);
             delete cursorNode;
             cursorNode = newNode;
-            appendChildNode(cursorNode);
-            cursorNode->setFlag(QSGNode::OwnedByParent);
+            if (cursorNode) {
+                appendChildNode(cursorNode);
+                cursorNode->setFlag(QSGNode::OwnedByParent);
+            }
         }
 
-        QSGSimpleRectNode *cursorNode;
+        QSGRectangleNode *cursorNode;
         QQuickTextNode* frameDecorationsNode;
 
     };
@@ -176,10 +172,17 @@ QQuickTextEdit::QQuickTextEdit(QQuickItem *parent)
     d->init();
 }
 
+QQuickTextEdit::QQuickTextEdit(QQuickTextEditPrivate &dd, QQuickItem *parent)
+: QQuickImplicitSizeItem(dd, parent)
+{
+    Q_D(QQuickTextEdit);
+    d->init();
+}
+
 QString QQuickTextEdit::text() const
 {
     Q_D(const QQuickTextEdit);
-    if (!d->textCached) {
+    if (!d->textCached && isComponentComplete()) {
         QQuickTextEditPrivate *d = const_cast<QQuickTextEditPrivate *>(d_func());
 #ifndef QT_NO_TEXTHTMLPARSER
         if (d->richText)
@@ -202,6 +205,17 @@ QString QQuickTextEdit::text() const
     If the family isn't available a family will be set using the font matching algorithm.
 */
 
+/*!
+    \qmlproperty string QtQuick::TextEdit::font.styleName
+    \since 5.6
+
+    Sets the style name of the font.
+
+    The style name is case insensitive. If set, the font will be matched against style name instead
+    of the font properties \l font.weight, \l font.bold and \l font.italic.
+*/
+
+
 /*!
     \qmlproperty bool QtQuick::TextEdit::font.bold
 
@@ -215,10 +229,14 @@ QString QQuickTextEdit::text() const
 
     The weight can be one of:
     \list
+    \li Font.Thin
     \li Font.Light
+    \li Font.ExtraLight
     \li Font.Normal - the default
+    \li Font.Medium
     \li Font.DemiBold
     \li Font.Bold
+    \li Font.ExtraBold
     \li Font.Black
     \endlist
 
@@ -418,9 +436,6 @@ void QQuickTextEdit::setTextFormat(TextFormat format)
     not require advanced features such as transformation of the text. Using such features in
     combination with the NativeRendering render type will lend poor and sometimes pixelated
     results.
-
-    On HighDpi "retina" displays and mobile and embedded platforms, this property is ignored
-    and QtRendering is always used.
 */
 QQuickTextEdit::RenderType QQuickTextEdit::renderType() const
 {
@@ -702,6 +717,75 @@ Qt::InputMethodHints QQuickTextEditPrivate::effectiveInputMethodHints() const
 }
 #endif
 
+void QQuickTextEditPrivate::setTopPadding(qreal value, bool reset)
+{
+    Q_Q(QQuickTextEdit);
+    qreal oldPadding = q->topPadding();
+    if (!reset || extra.isAllocated()) {
+        extra.value().topPadding = value;
+        extra.value().explicitTopPadding = !reset;
+    }
+    if ((!reset && !qFuzzyCompare(oldPadding, value)) || (reset && !qFuzzyCompare(oldPadding, padding()))) {
+        q->updateSize();
+        emit q->topPaddingChanged();
+    }
+}
+
+void QQuickTextEditPrivate::setLeftPadding(qreal value, bool reset)
+{
+    Q_Q(QQuickTextEdit);
+    qreal oldPadding = q->leftPadding();
+    if (!reset || extra.isAllocated()) {
+        extra.value().leftPadding = value;
+        extra.value().explicitLeftPadding = !reset;
+    }
+    if ((!reset && !qFuzzyCompare(oldPadding, value)) || (reset && !qFuzzyCompare(oldPadding, padding()))) {
+        q->updateSize();
+        emit q->leftPaddingChanged();
+    }
+}
+
+void QQuickTextEditPrivate::setRightPadding(qreal value, bool reset)
+{
+    Q_Q(QQuickTextEdit);
+    qreal oldPadding = q->rightPadding();
+    if (!reset || extra.isAllocated()) {
+        extra.value().rightPadding = value;
+        extra.value().explicitRightPadding = !reset;
+    }
+    if ((!reset && !qFuzzyCompare(oldPadding, value)) || (reset && !qFuzzyCompare(oldPadding, padding()))) {
+        q->updateSize();
+        emit q->rightPaddingChanged();
+    }
+}
+
+void QQuickTextEditPrivate::setBottomPadding(qreal value, bool reset)
+{
+    Q_Q(QQuickTextEdit);
+    qreal oldPadding = q->bottomPadding();
+    if (!reset || extra.isAllocated()) {
+        extra.value().bottomPadding = value;
+        extra.value().explicitBottomPadding = !reset;
+    }
+    if ((!reset && !qFuzzyCompare(oldPadding, value)) || (reset && !qFuzzyCompare(oldPadding, padding()))) {
+        q->updateSize();
+        emit q->bottomPaddingChanged();
+    }
+}
+
+bool QQuickTextEditPrivate::isImplicitResizeEnabled() const
+{
+    return !extra.isAllocated() || extra->implicitResize;
+}
+
+void QQuickTextEditPrivate::setImplicitResizeEnabled(bool enabled)
+{
+    if (!enabled)
+        extra.value().implicitResize = false;
+    else if (extra.isAllocated())
+        extra->implicitResize = true;
+}
+
 QQuickTextEdit::VAlignment QQuickTextEdit::vAlign() const
 {
     Q_D(const QQuickTextEdit);
@@ -830,7 +914,7 @@ void QQuickTextEdit::setBaseUrl(const QUrl &url)
     if (baseUrl() != url) {
         d->baseUrl = url;
 
-        d->document->setBaseUrl(url, d->richText);
+        d->document->setBaseUrl(url);
         emit baseUrlChanged();
     }
 }
@@ -1239,15 +1323,21 @@ void QQuickTextEdit::setTextMargin(qreal margin)
     \endlist
 */
 
-#ifndef QT_NO_IM
 Qt::InputMethodHints QQuickTextEdit::inputMethodHints() const
 {
+#ifdef QT_NO_IM
+    return Qt::ImhNone;
+#else
     Q_D(const QQuickTextEdit);
     return d->inputMethodHints;
+#endif // QT_NO_IM
 }
 
 void QQuickTextEdit::setInputMethodHints(Qt::InputMethodHints hints)
 {
+#ifdef QT_NO_IM
+    Q_UNUSED(hints);
+#else
     Q_D(QQuickTextEdit);
 
     if (hints == d->inputMethodHints)
@@ -1256,14 +1346,15 @@ void QQuickTextEdit::setInputMethodHints(Qt::InputMethodHints hints)
     d->inputMethodHints = hints;
     updateInputMethod(Qt::ImHints);
     emit inputMethodHintsChanged();
-}
 #endif // QT_NO_IM
+}
 
 void QQuickTextEdit::geometryChanged(const QRectF &newGeometry,
                                   const QRectF &oldGeometry)
 {
     Q_D(QQuickTextEdit);
-    if (newGeometry.width() != oldGeometry.width() && widthValid() && !d->inLayout) {
+    if (!d->inLayout && ((newGeometry.width() != oldGeometry.width() && widthValid())
+        || (newGeometry.height() != oldGeometry.height() && heightValid()))) {
         updateSize();
         updateWholeDocument();
         moveCursorDelegate();
@@ -1281,7 +1372,7 @@ void QQuickTextEdit::componentComplete()
     Q_D(QQuickTextEdit);
     QQuickImplicitSizeItem::componentComplete();
 
-    d->document->setBaseUrl(baseUrl(), d->richText);
+    d->document->setBaseUrl(baseUrl());
 #ifndef QT_NO_TEXTHTML_PARSER
     if (d->richText)
         d->control->setHtml(d->text);
@@ -1344,8 +1435,8 @@ void QQuickTextEdit::setSelectByKeyboard(bool on)
 
     If true, the user can use the mouse to select text in some
     platform-specific way. Note that for some platforms this may
-    not be an appropriate interaction (eg. may conflict with how
-    the text needs to behave inside a Flickable.
+    not be an appropriate interaction; it may conflict with how
+    the text needs to behave inside a Flickable, for example.
 */
 bool QQuickTextEdit::selectByMouse() const
 {
@@ -1432,6 +1523,11 @@ void QQuickTextEdit::setReadOnly(bool r)
     emit readOnlyChanged(r);
     if (!d->selectByKeyboardSet)
         emit selectByKeyboardChanged(!r);
+    if (r) {
+        setCursorVisible(false);
+    } else if (hasActiveFocus()) {
+        setCursorVisible(true);
+    }
 }
 
 bool QQuickTextEdit::isReadOnly() const
@@ -1649,6 +1745,8 @@ void QQuickTextEdit::mousePressEvent(QMouseEvent *event)
 #ifndef QT_NO_IM
         if (hasActiveFocus() && hadActiveFocus && !isReadOnly())
             qGuiApp->inputMethod()->show();
+#else
+        Q_UNUSED(hadActiveFocus);
 #endif
     }
     if (!event->isAccepted())
@@ -1709,9 +1807,9 @@ void QQuickTextEdit::inputMethodEvent(QInputMethodEvent *event)
 
 /*!
 \overload
-Returns the value of the given \a property.
+Returns the value of the given \a property and \a argument.
 */
-QVariant QQuickTextEdit::inputMethodQuery(Qt::InputMethodQuery property) const
+QVariant QQuickTextEdit::inputMethodQuery(Qt::InputMethodQuery property, QVariant argument) const
 {
     Q_D(const QQuickTextEdit);
 
@@ -1724,11 +1822,19 @@ QVariant QQuickTextEdit::inputMethodQuery(Qt::InputMethodQuery property) const
         v = (int)d->effectiveInputMethodHints();
         break;
     default:
-        v = d->control->inputMethodQuery(property);
+        v = d->control->inputMethodQuery(property, argument);
         break;
     }
     return v;
+}
 
+/*!
+\overload
+Returns the value of the given \a property.
+*/
+QVariant QQuickTextEdit::inputMethodQuery(Qt::InputMethodQuery property) const
+{
+    return inputMethodQuery(property, QVariant());
 }
 #endif // QT_NO_IM
 
@@ -1737,6 +1843,7 @@ void QQuickTextEdit::triggerPreprocess()
     Q_D(QQuickTextEdit);
     if (d->updateType == QQuickTextEditPrivate::UpdateNone)
         d->updateType = QQuickTextEditPrivate::UpdateOnlyPreprocess;
+    polish();
     update();
 }
 
@@ -1756,6 +1863,33 @@ static inline void updateNodeTransform(QQuickTextNode* node, const QPointF &topL
     node->setMatrix(transformMatrix);
 }
 
+/*!
+ * \internal
+ *
+ * Invalidates font caches owned by the text objects owned by the element
+ * to work around the fact that text objects cannot be used from multiple threads.
+ */
+void QQuickTextEdit::invalidateFontCaches()
+{
+    Q_D(QQuickTextEdit);
+    if (d->document == 0)
+        return;
+
+    QTextBlock block;
+    for (block = d->document->firstBlock(); block.isValid(); block = block.next()) {
+        if (block.layout() != 0 && block.layout()->engine() != 0)
+            block.layout()->engine()->resetFontEngineCache();
+    }
+}
+
+inline void resetEngine(QQuickTextNodeEngine *engine, const QColor& textColor, const QColor& selectedTextColor, const QColor& selectionColor)
+{
+    *engine = QQuickTextNodeEngine();
+    engine->setTextColor(textColor);
+    engine->setSelectedTextColor(selectedTextColor);
+    engine->setSelectionColor(selectionColor);
+}
+
 QSGNode *QQuickTextEdit::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *updatePaintNodeData)
 {
     Q_UNUSED(updatePaintNodeData);
@@ -1769,14 +1903,20 @@ QSGNode *QQuickTextEdit::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *
 
     d->updateType = QQuickTextEditPrivate::UpdateNone;
 
-    if (!oldNode) // If we had any text node references, they were deleted along with the root node
+    if (!oldNode) {
+        // If we had any QQuickTextNode node references, they were deleted along with the root node
+        // But here we must delete the Node structures in textNodeMap
+        qDeleteAll(d->textNodeMap);
         d->textNodeMap.clear();
+    }
 
     RootNode *rootNode = static_cast<RootNode *>(oldNode);
     TextNodeIterator nodeIterator = d->textNodeMap.begin();
     while (nodeIterator != d->textNodeMap.end() && !(*nodeIterator)->dirty())
         ++nodeIterator;
 
+    QQuickTextNodeEngine engine;
+    QQuickTextNodeEngine frameDecorationsEngine;
 
     if (!oldNode || nodeIterator < d->textNodeMap.end()) {
 
@@ -1796,12 +1936,17 @@ QSGNode *QQuickTextEdit::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *
 
         // FIXME: the text decorations could probably be handled separately (only updated for affected textFrames)
         rootNode->resetFrameDecorations(d->createTextNode());
+        resetEngine(&frameDecorationsEngine, d->color, d->selectedTextColor, d->selectionColor);
 
         QQuickTextNode *node = 0;
 
         int currentNodeSize = 0;
         int nodeStart = firstDirtyPos;
         QPointF basePosition(d->xoff, d->yoff);
+        QMatrix4x4 basePositionMatrix;
+        basePositionMatrix.translate(basePosition.x(), basePosition.y());
+        rootNode->setMatrix(basePositionMatrix);
+
         QPointF nodeOffset;
         TextNode *firstCleanNode = (nodeIterator != d->textNodeMap.end()) ? *nodeIterator : 0;
 
@@ -1811,12 +1956,12 @@ QSGNode *QQuickTextEdit::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *
         while (!frames.isEmpty()) {
             QTextFrame *textFrame = frames.takeFirst();
             frames.append(textFrame->childFrames());
-            rootNode->frameDecorationsNode->m_engine->addFrameDecorations(d->document, textFrame);
-
+            frameDecorationsEngine.addFrameDecorations(d->document, textFrame);
 
             if (textFrame->lastPosition() < firstDirtyPos || (firstCleanNode && textFrame->firstPosition() >= firstCleanNode->startPos()))
                 continue;
             node = d->createTextNode();
+            resetEngine(&engine, d->color, d->selectedTextColor, d->selectionColor);
 
             if (textFrame->firstPosition() > textFrame->lastPosition()
                     && textFrame->frameFormat().position() != QTextFrameFormat::InFlow) {
@@ -1825,17 +1970,10 @@ QSGNode *QQuickTextEdit::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *
                 ProtectedLayoutAccessor *a = static_cast<ProtectedLayoutAccessor *>(d->document->documentLayout());
                 QTextCharFormat format = a->formatAccessor(pos);
                 QTextBlock block = textFrame->firstCursorPosition().block();
-                node->m_engine->setCurrentLine(block.layout()->lineForTextPosition(pos - block.position()));
-                node->m_engine->addTextObject(QPointF(0, 0), format, QQuickTextNodeEngine::Unselected, d->document,
+                engine.setCurrentLine(block.layout()->lineForTextPosition(pos - block.position()));
+                engine.addTextObject(QPointF(0, 0), format, QQuickTextNodeEngine::Unselected, d->document,
                                               pos, textFrame->frameFormat().position());
                 nodeStart = pos;
-            } else if (qobject_cast<QTextTable*>(textFrame)) { // To keep things simple, map text tables as one text node
-                QTextFrame::iterator it = textFrame->begin();
-                nodeOffset =  d->document->documentLayout()->frameBoundingRect(textFrame).topLeft();
-                updateNodeTransform(node, nodeOffset);
-                while (!it.atEnd())
-                    node->m_engine->addTextBlock(d->document, (it++).currentBlock(), basePosition - nodeOffset, d->color, QColor(), selectionStart(), selectionEnd() - 1);
-                nodeStart = textFrame->firstPosition();
             } else {
                 // Having nodes spanning across frame boundaries will break the current bookkeeping mechanism. We need to prevent that.
                 QList<int> frameBoundaries;
@@ -1851,13 +1989,13 @@ QSGNode *QQuickTextEdit::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *
                     if (block.position() < firstDirtyPos)
                         continue;
 
-                    if (!node->m_engine->hasContents()) {
+                    if (!engine.hasContents()) {
                         nodeOffset = d->document->documentLayout()->blockBoundingRect(block).topLeft();
                         updateNodeTransform(node, nodeOffset);
                         nodeStart = block.position();
                     }
 
-                    node->m_engine->addTextBlock(d->document, block, basePosition - nodeOffset, d->color, QColor(), selectionStart(), selectionEnd() - 1);
+                    engine.addTextBlock(d->document, block, -nodeOffset, d->color, QColor(), selectionStart(), selectionEnd() - 1);
                     currentNodeSize += block.length();
 
                     if ((it.atEnd()) || (firstCleanNode && block.next().position() >= firstCleanNode->startPos())) // last node that needed replacing or last block of the frame
@@ -1866,15 +2004,16 @@ QSGNode *QQuickTextEdit::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *
                     QList<int>::const_iterator lowerBound = std::lower_bound(frameBoundaries.constBegin(), frameBoundaries.constEnd(), block.next().position());
                     if (currentNodeSize > nodeBreakingSize || lowerBound == frameBoundaries.constEnd() || *lowerBound > nodeStart) {
                         currentNodeSize = 0;
-                        d->addCurrentTextNodeToRoot(rootNode, node, nodeIterator, nodeStart);
+                        d->addCurrentTextNodeToRoot(&engine, rootNode, node, nodeIterator, nodeStart);
                         node = d->createTextNode();
+                        resetEngine(&engine, d->color, d->selectedTextColor, d->selectionColor);
                         nodeStart = block.next().position();
                     }
                 }
             }
-            d->addCurrentTextNodeToRoot(rootNode, node, nodeIterator, nodeStart);
+            d->addCurrentTextNodeToRoot(&engine, rootNode, node, nodeIterator, nodeStart);
         }
-        rootNode->frameDecorationsNode->m_engine->addToSceneGraph(rootNode->frameDecorationsNode, QQuickText::Normal, QColor());
+        frameDecorationsEngine.addToSceneGraph(rootNode->frameDecorationsNode, QQuickText::Normal, QColor());
         // Now prepend the frame decorations since we want them rendered first, with the text nodes and cursor in front.
         rootNode->prependChildNode(rootNode->frameDecorationsNode);
 
@@ -1898,16 +2037,23 @@ QSGNode *QQuickTextEdit::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *
         std::sort(d->textNodeMap.begin(), d->textNodeMap.end(), &comesBefore);
     }
 
-    if (d->cursorComponent == 0 && !isReadOnly()) {
-        QColor color = (!d->cursorVisible || !d->control->cursorOn())
-                ? QColor(0, 0, 0, 0)
-                : d->color;
-        rootNode->resetCursorNode(new QSGSimpleRectNode(cursorRectangle(), color));
+    if (d->cursorComponent == 0) {
+        QSGRectangleNode* cursor = 0;
+        if (!isReadOnly() && d->cursorVisible && d->control->cursorOn())
+            cursor = d->sceneGraphContext()->createRectangleNode(d->control->cursorRect(), d->color);
+        rootNode->resetCursorNode(cursor);
     }
 
+    invalidateFontCaches();
+
     return rootNode;
 }
 
+void QQuickTextEdit::updatePolish()
+{
+    invalidateFontCaches();
+}
+
 /*!
     \qmlproperty bool QtQuick::TextEdit::canPaste
 
@@ -1950,7 +2096,6 @@ bool QQuickTextEdit::canRedo() const
     return d->document->isRedoAvailable();
 }
 
-#ifndef QT_NO_IM
 /*!
     \qmlproperty bool QtQuick::TextEdit::inputMethodComposing
 
@@ -1965,10 +2110,27 @@ bool QQuickTextEdit::canRedo() const
 */
 bool QQuickTextEdit::isInputMethodComposing() const
 {
+#ifdef QT_NO_IM
+    return false;
+#else
     Q_D(const QQuickTextEdit);
     return d->control->hasImState();
-}
 #endif // QT_NO_IM
+}
+
+QQuickTextEditPrivate::ExtraData::ExtraData()
+    : padding(0)
+    , topPadding(0)
+    , leftPadding(0)
+    , rightPadding(0)
+    , bottomPadding(0)
+    , explicitTopPadding(false)
+    , explicitLeftPadding(false)
+    , explicitRightPadding(false)
+    , explicitBottomPadding(false)
+    , implicitResize(true)
+{
+}
 
 void QQuickTextEditPrivate::init()
 {
@@ -2011,6 +2173,7 @@ void QQuickTextEditPrivate::init()
     qmlobject_connect(document, QQuickTextDocumentWithImageResources, SIGNAL(redoAvailable(bool)), q, QQuickTextEdit, SIGNAL(canRedoChanged()));
     qmlobject_connect(document, QQuickTextDocumentWithImageResources, SIGNAL(imagesLoaded()), q, QQuickTextEdit, SLOT(updateSize()));
     QObject::connect(document, &QQuickTextDocumentWithImageResources::contentsChange, q, &QQuickTextEdit::q_contentsChange);
+    QObject::connect(document->documentLayout(), &QAbstractTextDocumentLayout::updateBlock, q, &QQuickTextEdit::invalidateBlock);
 
     document->setDefaultFont(font);
     document->setDocumentMargin(textMargin);
@@ -2032,7 +2195,6 @@ void QQuickTextEdit::q_textChanged()
     d->determineHorizontalAlignment();
     d->updateDefaultTextOption();
     updateSize();
-    updateTotalLines();
     emit textChanged();
 }
 
@@ -2053,7 +2215,7 @@ void QQuickTextEdit::markDirtyNodesForRange(int start, int end, int charDelta)
     }
 
     // mark the affected nodes as dirty
-    while (it != d->textNodeMap.constEnd()) {
+    while (it != d->textNodeMap.end()) {
         if ((*it)->startPos() <= end)
             (*it)->setDirty();
         else if (charDelta)
@@ -2073,6 +2235,7 @@ void QQuickTextEdit::q_contentsChange(int pos, int charsRemoved, int charsAdded)
 
     markDirtyNodesForRange(pos, editRange, delta);
 
+    polish();
     if (isComponentComplete()) {
         d->updateType = QQuickTextEditPrivate::UpdatePaintNode;
         update();
@@ -2100,6 +2263,7 @@ void QQuickTextEdit::updateSelection()
     // No need for node updates when we go from an empty selection to another empty selection
     if (d->control->textCursor().hasSelection() || d->hadSelection) {
         markDirtyNodesForRange(qMin(d->lastSelectionStart, d->control->textCursor().selectionStart()), qMax(d->control->textCursor().selectionEnd(), d->lastSelectionEnd), 0);
+        polish();
         if (isComponentComplete()) {
             d->updateType = QQuickTextEditPrivate::UpdatePaintNode;
             update();
@@ -2177,7 +2341,7 @@ void QQuickTextEdit::updateSize()
         return;
     }
 
-    qreal naturalWidth = d->implicitWidth;
+    qreal naturalWidth = d->implicitWidth - leftPadding() - rightPadding();
 
     qreal newWidth = d->document->idealWidth();
     // ### assumes that if the width is set, the text will fill to edges
@@ -2195,13 +2359,14 @@ void QQuickTextEdit::updateSize()
 
             const bool wasInLayout = d->inLayout;
             d->inLayout = true;
-            setImplicitWidth(naturalWidth);
+            if (d->isImplicitResizeEnabled())
+                setImplicitWidth(naturalWidth + leftPadding() + rightPadding());
             d->inLayout = wasInLayout;
             if (d->inLayout)    // probably the result of a binding loop, but by letting it
                 return;         // get this far we'll get a warning to that effect.
         }
         if (d->document->textWidth() != width()) {
-            d->document->setTextWidth(width());
+            d->document->setTextWidth(width() - leftPadding() - rightPadding());
             newWidth = d->document->idealWidth();
         }
         //### need to confirm cost of always setting these
@@ -2214,20 +2379,23 @@ void QQuickTextEdit::updateSize()
     QFontMetricsF fm(d->font);
     qreal newHeight = d->document->isEmpty() ? qCeil(fm.height()) : d->document->size().height();
 
-    // ### Setting the implicitWidth triggers another updateSize(), and unless there are bindings nothing has changed.
-    if (!widthValid() && !d->requireImplicitWidth)
-        setImplicitSize(newWidth, newHeight);
-    else
-        setImplicitHeight(newHeight);
+    if (d->isImplicitResizeEnabled()) {
+        // ### Setting the implicitWidth triggers another updateSize(), and unless there are bindings nothing has changed.
+        if (!widthValid() && !d->requireImplicitWidth)
+            setImplicitSize(newWidth + leftPadding() + rightPadding(), newHeight + topPadding() + bottomPadding());
+        else
+            setImplicitHeight(newHeight + topPadding() + bottomPadding());
+    }
 
-    d->xoff = qMax(qreal(0), QQuickTextUtil::alignedX(d->document->size().width(), width(), effectiveHAlign()));
-    d->yoff = QQuickTextUtil::alignedY(d->document->size().height(), height(), d->vAlign);
+    d->xoff = leftPadding() + qMax(qreal(0), QQuickTextUtil::alignedX(d->document->size().width(), width() - leftPadding() - rightPadding(), effectiveHAlign()));
+    d->yoff = topPadding() + QQuickTextUtil::alignedY(d->document->size().height(), height() - topPadding() - bottomPadding(), d->vAlign);
     setBaselineOffset(fm.ascent() + d->yoff + d->textMargin);
 
     QSizeF size(newWidth, newHeight);
     if (d->contentSize != size) {
         d->contentSize = size;
         emit contentSizeChanged();
+        updateTotalLines();
     }
 }
 
@@ -2239,6 +2407,19 @@ void QQuickTextEdit::updateWholeDocument()
             node->setDirty();
     }
 
+    polish();
+    if (isComponentComplete()) {
+        d->updateType = QQuickTextEditPrivate::UpdatePaintNode;
+        update();
+    }
+}
+
+void QQuickTextEdit::invalidateBlock(const QTextBlock &block)
+{
+    Q_D(QQuickTextEdit);
+    markDirtyNodesForRange(block.position(), block.position() + block.length(), 0);
+
+    polish();
     if (isComponentComplete()) {
         d->updateType = QQuickTextEditPrivate::UpdatePaintNode;
         update();
@@ -2248,6 +2429,7 @@ void QQuickTextEdit::updateWholeDocument()
 void QQuickTextEdit::updateCursor()
 {
     Q_D(QQuickTextEdit);
+    polish();
     if (isComponentComplete()) {
         d->updateType = QQuickTextEditPrivate::UpdatePaintNode;
         update();
@@ -2343,25 +2525,29 @@ void QQuickTextEditPrivate::handleFocusEvent(QFocusEvent *event)
 {
     Q_Q(QQuickTextEdit);
     bool focus = event->type() == QEvent::FocusIn;
-    q->setCursorVisible(focus);
+    if (!q->isReadOnly())
+        q->setCursorVisible(focus);
     control->processEvent(event, QPointF(-xoff, -yoff));
     if (focus) {
         q->q_updateAlignment();
 #ifndef QT_NO_IM
         if (focusOnPress && !q->isReadOnly())
             qGuiApp->inputMethod()->show();
-        q->connect(qApp->inputMethod(), SIGNAL(inputDirectionChanged(Qt::LayoutDirection)),
+        q->connect(QGuiApplication::inputMethod(), SIGNAL(inputDirectionChanged(Qt::LayoutDirection)),
                 q, SLOT(q_updateAlignment()));
+#endif
     } else {
-        q->disconnect(qApp->inputMethod(), SIGNAL(inputDirectionChanged(Qt::LayoutDirection)),
+#ifndef QT_NO_IM
+        q->disconnect(QGuiApplication::inputMethod(), SIGNAL(inputDirectionChanged(Qt::LayoutDirection)),
                    q, SLOT(q_updateAlignment()));
 #endif
+        emit q->editingFinished();
     }
 }
 
-void QQuickTextEditPrivate::addCurrentTextNodeToRoot(QSGTransformNode *root, QQuickTextNode *node, TextNodeIterator &it, int startPos)
+void QQuickTextEditPrivate::addCurrentTextNodeToRoot(QQuickTextNodeEngine *engine, QSGTransformNode *root, QQuickTextNode *node, TextNodeIterator &it, int startPos)
 {
-    node->m_engine->addToSceneGraph(node, QQuickText::Normal, QColor());
+    engine->addToSceneGraph(node, QQuickText::Normal, QColor());
     it = textNodeMap.insert(it, new TextNode(startPos, node));
     ++it;
     root->appendChildNode(node);
@@ -2371,8 +2557,7 @@ QQuickTextNode *QQuickTextEditPrivate::createTextNode()
 {
     Q_Q(QQuickTextEdit);
     QQuickTextNode* node = new QQuickTextNode(q);
-    node->setUseNativeRenderer(renderType == QQuickTextEdit::NativeRendering && window->devicePixelRatio() <= 1);
-    node->initEngine(color, selectedTextColor, selectionColor);
+    node->setUseNativeRenderer(renderType == QQuickTextEdit::NativeRendering);
     return node;
 }
 
@@ -2511,25 +2696,36 @@ bool QQuickTextEditPrivate::isLinkHoveredConnected()
 }
 
 /*!
-    \qmlsignal QtQuick::TextEdit::onLinkHovered(string link)
+    \qmlsignal QtQuick::TextEdit::linkHovered(string link)
     \since 5.2
 
-    This handler is called when the user hovers a link embedded in the text.
+    This signal is emitted when the user hovers a link embedded in the text.
     The link must be in rich text or HTML format and the
     \a link string provides access to the particular link.
 
-    \sa hoveredLink
+    The corresponding handler is \c onLinkHovered.
+
+    \sa hoveredLink, linkAt()
+*/
+
+/*!
+    \qmlsignal QtQuick::TextEdit::editingFinished()
+    \since 5.6
+
+    This signal is emitted when the text edit loses focus.
+
+    The corresponding handler is \c onEditingFinished.
 */
 
 /*!
     \qmlproperty string QtQuick::TextEdit::hoveredLink
     \since 5.2
 
-    This property contains the link string when user hovers a link
+    This property contains the link string when the user hovers a link
     embedded in the text. The link must be in rich text or HTML format
     and the link string provides access to the particular link.
 
-    \sa onLinkHovered
+    \sa linkHovered, linkAt()
 */
 
 QString QQuickTextEdit::hoveredLink() const
@@ -2602,4 +2798,144 @@ void QQuickTextEdit::append(const QString &text)
     d->control->updateCursorRectangle(false);
 }
 
+/*!
+    \qmlmethod QtQuick::TextEdit::linkAt(real x, real y)
+    \since 5.3
+
+    Returns the link string at point \a x, \a y in content coordinates,
+    or an empty string if no link exists at that point.
+
+    \sa hoveredLink
+*/
+QString QQuickTextEdit::linkAt(qreal x, qreal y) const
+{
+    Q_D(const QQuickTextEdit);
+    return d->control->anchorAt(QPointF(x + topPadding(), y + leftPadding()));
+}
+
+/*!
+    \since 5.6
+    \qmlproperty real QtQuick::TextEdit::padding
+    \qmlproperty real QtQuick::TextEdit::topPadding
+    \qmlproperty real QtQuick::TextEdit::leftPadding
+    \qmlproperty real QtQuick::TextEdit::bottomPadding
+    \qmlproperty real QtQuick::TextEdit::rightPadding
+
+    These properties hold the padding around the content. This space is reserved
+    in addition to the contentWidth and contentHeight.
+*/
+qreal QQuickTextEdit::padding() const
+{
+    Q_D(const QQuickTextEdit);
+    return d->padding();
+}
+
+void QQuickTextEdit::setPadding(qreal padding)
+{
+    Q_D(QQuickTextEdit);
+    if (qFuzzyCompare(d->padding(), padding))
+        return;
+
+    d->extra.value().padding = padding;
+    updateSize();
+    if (isComponentComplete()) {
+        d->updateType = QQuickTextEditPrivate::UpdatePaintNode;
+        update();
+    }
+    emit paddingChanged();
+    if (!d->extra.isAllocated() || !d->extra->explicitTopPadding)
+        emit topPaddingChanged();
+    if (!d->extra.isAllocated() || !d->extra->explicitLeftPadding)
+        emit leftPaddingChanged();
+    if (!d->extra.isAllocated() || !d->extra->explicitRightPadding)
+        emit rightPaddingChanged();
+    if (!d->extra.isAllocated() || !d->extra->explicitBottomPadding)
+        emit bottomPaddingChanged();
+}
+
+void QQuickTextEdit::resetPadding()
+{
+    setPadding(0);
+}
+
+qreal QQuickTextEdit::topPadding() const
+{
+    Q_D(const QQuickTextEdit);
+    if (d->extra.isAllocated() && d->extra->explicitTopPadding)
+        return d->extra->topPadding;
+    return d->padding();
+}
+
+void QQuickTextEdit::setTopPadding(qreal padding)
+{
+    Q_D(QQuickTextEdit);
+    d->setTopPadding(padding);
+}
+
+void QQuickTextEdit::resetTopPadding()
+{
+    Q_D(QQuickTextEdit);
+    d->setTopPadding(0, true);
+}
+
+qreal QQuickTextEdit::leftPadding() const
+{
+    Q_D(const QQuickTextEdit);
+    if (d->extra.isAllocated() && d->extra->explicitLeftPadding)
+        return d->extra->leftPadding;
+    return d->padding();
+}
+
+void QQuickTextEdit::setLeftPadding(qreal padding)
+{
+    Q_D(QQuickTextEdit);
+    d->setLeftPadding(padding);
+}
+
+void QQuickTextEdit::resetLeftPadding()
+{
+    Q_D(QQuickTextEdit);
+    d->setLeftPadding(0, true);
+}
+
+qreal QQuickTextEdit::rightPadding() const
+{
+    Q_D(const QQuickTextEdit);
+    if (d->extra.isAllocated() && d->extra->explicitRightPadding)
+        return d->extra->rightPadding;
+    return d->padding();
+}
+
+void QQuickTextEdit::setRightPadding(qreal padding)
+{
+    Q_D(QQuickTextEdit);
+    d->setRightPadding(padding);
+}
+
+void QQuickTextEdit::resetRightPadding()
+{
+    Q_D(QQuickTextEdit);
+    d->setRightPadding(0, true);
+}
+
+qreal QQuickTextEdit::bottomPadding() const
+{
+    Q_D(const QQuickTextEdit);
+    if (d->extra.isAllocated() && d->extra->explicitBottomPadding)
+        return d->extra->bottomPadding;
+    return d->padding();
+}
+
+void QQuickTextEdit::setBottomPadding(qreal padding)
+{
+    Q_D(QQuickTextEdit);
+    d->setBottomPadding(padding);
+}
+
+void QQuickTextEdit::resetBottomPadding()
+{
+    Q_D(QQuickTextEdit);
+    d->setBottomPadding(0, true);
+}
+
 QT_END_NAMESPACE
