@@ -1,46 +1,49 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
-#ifndef ULTRARENDERER_H
-#define ULTRARENDERER_H
+#ifndef QSGBATCHRENDERER_P_H
+#define QSGBATCHRENDERER_P_H
+
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
 
 #include <private/qsgrenderer_p.h>
 #include <private/qsgnodeupdater_p.h>
@@ -48,6 +51,8 @@
 
 #include <private/qsgrendernode_p.h>
 
+#include <QtCore/QBitArray>
+
 QT_BEGIN_NAMESPACE
 
 class QOpenGLVertexArrayObject;
@@ -67,6 +72,138 @@ class Updater;
 class Renderer;
 class ShaderManager;
 
+template <typename Type, int PageSize> class AllocatorPage
+{
+public:
+    // The memory used by this allocator
+    char data[sizeof(Type) * PageSize];
+
+    // 'blocks' contains a list of free indices which can be allocated.
+    // The first available index is found in PageSize - available.
+    int blocks[PageSize];
+
+    // 'available' is the number of available instances this page has left to allocate.
+    int available;
+
+    // This is not strictly needed, but useful for sanity checking and anyway
+    // pretty small..
+    QBitArray allocated;
+
+    AllocatorPage()
+        : available(PageSize)
+        , allocated(PageSize)
+    {
+        for (int i=0; i<PageSize; ++i) blocks[i] = i;
+    }
+
+    const Type *at(uint index) const
+    {
+        return (Type *) &data[index * sizeof(Type)];
+    }
+
+    Type *at(uint index)
+    {
+        return (Type *) &data[index * sizeof(Type)];
+    }
+};
+
+template <typename Type, int PageSize> class Allocator
+{
+public:
+    Allocator()
+        : m_freePage(0)
+    {
+        pages.push_back(new AllocatorPage<Type, PageSize>());
+    }
+
+    ~Allocator()
+    {
+        qDeleteAll(pages);
+    }
+
+    Type *allocate()
+    {
+        AllocatorPage<Type, PageSize> *p = 0;
+        for (int i = m_freePage; i < pages.size(); i++) {
+            if (pages.at(i)->available > 0) {
+                p = pages.at(i);
+                m_freePage = i;
+                break;
+            }
+        }
+
+        // we couldn't find a free page from m_freePage to the last page.
+        // either there is no free pages, or there weren't any in the area we
+        // scanned: rescanning is expensive, so let's just assume there isn't
+        // one. when an item is released, we'll reset m_freePage anyway.
+        if (!p) {
+            p = new AllocatorPage<Type, PageSize>();
+            m_freePage = pages.count();
+            pages.push_back(p);
+        }
+        uint pos = p->blocks[PageSize - p->available];
+        void *mem = p->at(pos);
+        p->available--;
+        p->allocated.setBit(pos);
+        Type *t = new (mem) Type();
+        return t;
+    }
+
+    void releaseExplicit(uint pageIndex, uint index)
+    {
+        AllocatorPage<Type, PageSize> *page = pages.at(pageIndex);
+        if (!page->allocated.testBit(index))
+            qFatal("Double delete in allocator: page=%d, index=%d", pageIndex , index);
+
+        // Call the destructor
+        page->at(index)->~Type();
+
+        page->allocated[index] = false;
+        page->available++;
+        page->blocks[PageSize - page->available] = index;
+
+        // Remove the pages if they are empty and they are the last ones. We need to keep the
+        // order of pages since we have references to their index, so we can only remove
+        // from the end.
+        while (page->available == PageSize && pages.size() > 1 && pages.back() == page) {
+            pages.pop_back();
+            delete page;
+            page = pages.back();
+        }
+
+        // Reset the free page to force a scan for a new free point.
+        m_freePage = 0;
+    }
+
+    void release(Type *t)
+    {
+        int pageIndex = -1;
+        for (int i=0; i<pages.size(); ++i) {
+            AllocatorPage<Type, PageSize> *p = pages.at(i);
+            if ((Type *) (&p->data[0]) <= t && (Type *) (&p->data[PageSize * sizeof(Type)]) > t) {
+                pageIndex = i;
+                break;
+            }
+        }
+        Q_ASSERT(pageIndex >= 0);
+
+        AllocatorPage<Type, PageSize> *page = pages.at(pageIndex);
+        int index = (quint64(t) - quint64(&page->data[0])) / sizeof(Type);
+
+        releaseExplicit(pageIndex, index);
+    }
+
+    QVector<AllocatorPage<Type, PageSize> *> pages;
+    int m_freePage;
+};
+
+
+inline bool hasMaterialWithBlending(QSGGeometryNode *n)
+{
+    return (n->opaqueMaterial() ? n->opaqueMaterial()->flags() & QSGMaterial::Blending
+                                : n->material()->flags() & QSGMaterial::Blending);
+}
+
 struct Pt {
     float x, y;
 
@@ -146,13 +283,15 @@ inline QDebug operator << (QDebug d, const Rect &r) {
 struct Buffer {
     GLuint id;
     int size;
+    // Data is only valid while preparing the upload. Exception is if we are using the
+    // broken IBO workaround or we are using a visualization mode.
     char *data;
 };
 
 struct Element {
 
-    Element(QSGGeometryNode *n)
-        : node(n)
+    Element()
+        : node(0)
         , batch(0)
         , nextInBatch(0)
         , root(0)
@@ -163,9 +302,15 @@ struct Element {
         , removed(false)
         , orphaned(false)
         , isRenderNode(false)
+        , isMaterialBlended(false)
     {
     }
 
+    void setNode(QSGGeometryNode *n) {
+        node = n;
+        isMaterialBlended = hasMaterialWithBlending(n);
+    }
+
     inline void ensureBoundsValid() {
         if (!boundsComputed)
             computeBounds();
@@ -187,13 +332,13 @@ struct Element {
     uint removed : 1;
     uint orphaned : 1;
     uint isRenderNode : 1;
+    uint isMaterialBlended : 1;
 };
 
 struct RenderNodeElement : public Element {
 
     RenderNodeElement(QSGRenderNode *rn)
-        : Element(0)
-        , renderNode(rn)
+        : renderNode(rn)
     {
         isRenderNode = true;
     }
@@ -233,7 +378,6 @@ struct DrawSet
 
 enum BatchCompatibility
 {
-    BatchBreaksOnBlending,
     BatchBreaksOnCompare,
     BatchIsCompatible
 };
@@ -290,15 +434,14 @@ struct Batch
 
 struct Node
 {
-    Node(QSGNode *node, Node *sparent = 0)
-        : sgNode(node)
-        , parent(sparent)
+    Node()
+        : sgNode(0)
+        , parent(0)
         , data(0)
         , dirtyState(0)
         , isOpaque(false)
         , isBatchRoot(false)
     {
-
     }
 
     QSGNode *sgNode;
@@ -378,10 +521,10 @@ public:
         float lastOpacity;
     };
 
-    ShaderManager(QSGRenderContext *ctx) : blitProgram(0), visualizeProgram(0), context(ctx) { }
+    ShaderManager(QSGRenderContext *ctx) : visualizeProgram(0), blitProgram(0), context(ctx) { }
     ~ShaderManager() {
-        qDeleteAll(rewrittenShaders.values());
-        qDeleteAll(stockShaders.values());
+        qDeleteAll(rewrittenShaders);
+        qDeleteAll(stockShaders);
     }
 
 public Q_SLOTS:
@@ -391,15 +534,17 @@ public:
     Shader *prepareMaterial(QSGMaterial *material);
     Shader *prepareMaterialNoRewrite(QSGMaterial *material);
 
+    QOpenGLShaderProgram *visualizeProgram;
+
+private:
     QHash<QSGMaterialType *, Shader *> rewrittenShaders;
     QHash<QSGMaterialType *, Shader *> stockShaders;
 
     QOpenGLShaderProgram *blitProgram;
-    QOpenGLShaderProgram *visualizeProgram;
     QSGRenderContext *context;
 };
 
-class Q_QUICK_PRIVATE_EXPORT Renderer : public QSGRenderer
+class Q_QUICK_PRIVATE_EXPORT Renderer : public QSGRenderer, public QOpenGLFunctions
 {
 public:
     Renderer(QSGRenderContext *);
@@ -414,11 +559,18 @@ public:
     };
 
 protected:
-    void nodeChanged(QSGNode *node, QSGNode::DirtyState state);
-    void preprocess() Q_DECL_OVERRIDE;
-    void render();
+    void nodeChanged(QSGNode *node, QSGNode::DirtyState state) Q_DECL_OVERRIDE;
+    void render() Q_DECL_OVERRIDE;
 
 private:
+    enum ClipTypeBit
+    {
+        NoClip = 0x00,
+        ScissorClip = 0x01,
+        StencilClip = 0x02
+    };
+    Q_DECLARE_FLAGS(ClipType, ClipTypeBit)
+
     enum RebuildFlag {
         BuildRenderListsForTaggedRoots      = 0x0001,
         BuildRenderLists                    = 0x0002,
@@ -428,8 +580,7 @@ private:
 
     friend class Updater;
 
-
-    void map(Buffer *buffer, int size);
+    void map(Buffer *buffer, int size, bool isIndexBuf = false);
     void unmap(Buffer *buffer, bool isIndexBuf = false);
 
     void buildRenderListsFromScratch();
@@ -450,6 +601,7 @@ private:
     void renderBatches();
     void renderMergedBatch(const Batch *batch);
     void renderUnmergedBatch(const Batch *batch);
+    ClipType updateStencilClip(const QSGClipNode *clip);
     void updateClip(const QSGClipNode *clipList, const Batch *batch);
     const QMatrix4x4 &matrixForRoot(Node *node);
     void renderRenderNode(Batch *batch);
@@ -464,6 +616,7 @@ private:
     void nodeWasRemoved(Node *node);
     void nodeWasAdded(QSGNode *node, Node *shadowParent);
     BatchRootInfo *batchRootInfo(Node *node);
+    void updateLineWidth(QSGGeometry *g);
 
     inline Batch *newBatch();
     void invalidateAndRecycleBatch(Batch *b);
@@ -476,7 +629,7 @@ private:
     void visualizeOverdraw();
     void visualizeOverdraw_helper(Node *node);
     void visualizeDrawGeometry(const QSGGeometry *g);
-    void setCustomRenderMode(const QByteArray &mode);
+    void setCustomRenderMode(const QByteArray &mode) Q_DECL_OVERRIDE;
 
     QSet<Node *> m_taggedRoots;
     QDataBuffer<Element *> m_opaqueRenderList;
@@ -511,14 +664,26 @@ private:
     QSGMaterial *m_currentMaterial;
     QSGMaterialShader *m_currentProgram;
     ShaderManager::Shader *m_currentShader;
+
+    QRect m_currentScissorRect;
+    int m_currentStencilValue;
+    QOpenGLShaderProgram m_clipProgram;
+    int m_clipMatrixId;
     const QSGClipNode *m_currentClip;
     ClipType m_currentClipType;
 
+    QDataBuffer<char> m_vertexUploadPool;
+#ifdef QSG_SEPARATE_INDEX_BUFFER
+    QDataBuffer<char> m_indexUploadPool;
+#endif
     // For minimal OpenGL core profile support
     QOpenGLVertexArrayObject *m_vao;
 
     QHash<Node *, uint> m_visualizeChanceSet;
     VisualizeMode m_visualizeMode;
+
+    Allocator<Node, 256> m_nodeAllocator;
+    Allocator<Element, 64> m_elementAllocator;
 };
 
 Batch *Renderer::newBatch()
@@ -543,4 +708,4 @@ Batch *Renderer::newBatch()
 
 QT_END_NAMESPACE
 
-#endif // ULTRARENDERER_H
+#endif // QSGBATCHRENDERER_P_H
