@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -86,7 +78,7 @@ QQuickStateActionEvent::~QQuickStateActionEvent()
 {
 }
 
-void QQuickStateActionEvent::execute(Reason)
+void QQuickStateActionEvent::execute()
 {
 }
 
@@ -95,7 +87,7 @@ bool QQuickStateActionEvent::isReversable()
     return false;
 }
 
-void QQuickStateActionEvent::reverse(Reason)
+void QQuickStateActionEvent::reverse()
 {
 }
 
@@ -364,8 +356,7 @@ void QQuickState::cancel()
 void QQuickStateAction::deleteFromBinding()
 {
     if (fromBinding) {
-        QQmlPropertyPrivate::setBinding(property, 0);
-        fromBinding->destroy();
+        QQmlPropertyPrivate::removeBinding(property);
         fromBinding = 0;
     }
 }
@@ -416,9 +407,6 @@ bool QQuickState::changeBindingInRevertList(QObject *target, const QString &name
         while (revertListIterator.hasNext()) {
             QQuickSimpleAction &simpleAction = revertListIterator.next();
             if (simpleAction.specifiedObject() == target && simpleAction.specifiedProperty() == name) {
-                if (simpleAction.binding())
-                    simpleAction.binding()->destroy();
-
                 simpleAction.setBinding(binding);
                 return true;
             }
@@ -438,15 +426,11 @@ bool QQuickState::removeEntryFromRevertList(QObject *target, const QString &name
         while (revertListIterator.hasNext()) {
             QQuickSimpleAction &simpleAction = revertListIterator.next();
             if (simpleAction.property().object() == target && simpleAction.property().name() == name) {
-                QQmlAbstractBinding *oldBinding = QQmlPropertyPrivate::binding(simpleAction.property());
-                if (oldBinding) {
-                    QQmlPropertyPrivate::setBinding(simpleAction.property(), 0);
-                    oldBinding->destroy();
-                }
+                QQmlPropertyPrivate::removeBinding(simpleAction.property());
 
                 simpleAction.property().write(simpleAction.value());
                 if (simpleAction.binding())
-                    QQmlPropertyPrivate::setBinding(simpleAction.property(), simpleAction.binding());
+                    QQmlPropertyPrivate::setBinding(simpleAction.binding());
 
                 revertListIterator.remove();
                 return true;
@@ -476,15 +460,11 @@ void QQuickState::removeAllEntriesFromRevertList(QObject *target)
          while (revertListIterator.hasNext()) {
              QQuickSimpleAction &simpleAction = revertListIterator.next();
              if (simpleAction.property().object() == target) {
-                 QQmlAbstractBinding *oldBinding = QQmlPropertyPrivate::binding(simpleAction.property());
-                 if (oldBinding) {
-                     QQmlPropertyPrivate::setBinding(simpleAction.property(), 0);
-                     oldBinding->destroy();
-                 }
+                 QQmlPropertyPrivate::removeBinding(simpleAction.property());
 
                  simpleAction.property().write(simpleAction.value());
                  if (simpleAction.binding())
-                     QQmlPropertyPrivate::setBinding(simpleAction.property(), simpleAction.binding());
+                     QQmlPropertyPrivate::setBinding(simpleAction.binding());
 
                  revertListIterator.remove();
              }
@@ -497,18 +477,15 @@ void QQuickState::addEntriesToRevertList(const QList<QQuickStateAction> &actionL
     Q_D(QQuickState);
     if (isStateActive()) {
         QList<QQuickSimpleAction> simpleActionList;
+        simpleActionList.reserve(actionList.count());
 
         QListIterator<QQuickStateAction> actionListIterator(actionList);
         while(actionListIterator.hasNext()) {
             const QQuickStateAction &action = actionListIterator.next();
             QQuickSimpleAction simpleAction(action);
             action.property.write(action.toValue);
-            if (!action.toBinding.isNull()) {
-                QQmlAbstractBinding *oldBinding = QQmlPropertyPrivate::binding(simpleAction.property());
-                if (oldBinding)
-                    QQmlPropertyPrivate::setBinding(simpleAction.property(), 0);
-                QQmlPropertyPrivate::setBinding(simpleAction.property(), action.toBinding.data(), QQmlPropertyPrivate::DontRemoveBinding);
-            }
+            if (action.toBinding)
+                QQmlPropertyPrivate::setBinding(action.toBinding.data());
 
             simpleActionList.append(simpleAction);
         }
@@ -622,7 +599,7 @@ void QQuickState::apply(QQuickTransition *trans, QQuickState *revert)
             for (int jj = 0; jj < d->revertList.count(); ++jj) {
                 if (d->revertList.at(jj).property() == action.property) {
                     found = true;
-                    if (d->revertList.at(jj).binding() != action.fromBinding) {
+                    if (d->revertList.at(jj).binding() != action.fromBinding.data()) {
                         action.deleteFromBinding();
                     }
                     break;
@@ -666,16 +643,13 @@ void QQuickState::apply(QQuickTransition *trans, QQuickState *revert)
         }
         if (!found) {
             QVariant cur = d->revertList.at(ii).property().read();
-            QQmlAbstractBinding *delBinding =
-                QQmlPropertyPrivate::setBinding(d->revertList.at(ii).property(), 0);
-            if (delBinding)
-                delBinding->destroy();
+            QQmlPropertyPrivate::removeBinding(d->revertList.at(ii).property());
 
             QQuickStateAction a;
             a.property = d->revertList.at(ii).property();
             a.fromValue = cur;
             a.toValue = d->revertList.at(ii).value();
-            a.toBinding = QQmlAbstractBinding::getPointer(d->revertList.at(ii).binding());
+            a.toBinding = d->revertList.at(ii).binding();
             a.specifiedObject = d->revertList.at(ii).specifiedObject();
             a.specifiedProperty = d->revertList.at(ii).specifiedProperty();
             a.event = d->revertList.at(ii).event();
