@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -190,7 +182,7 @@ void QQuickStyledText::parse(const QString &string, QTextLayout &layout,
 
 void QQuickStyledTextPrivate::parse()
 {
-    QList<QTextLayout::FormatRange> ranges;
+    QVector<QTextLayout::FormatRange> ranges;
     QStack<QTextCharFormat> formatStack;
 
     QString drawText;
@@ -291,7 +283,7 @@ void QQuickStyledTextPrivate::parse()
     }
 
     layout.setText(drawText);
-    layout.setAdditionalFormats(ranges);
+    layout.setFormats(ranges);
 }
 
 void QQuickStyledTextPrivate::appendText(const QString &textIn, int start, int length, QString &textOut)
@@ -550,6 +542,12 @@ void QQuickStyledTextPrivate::parseEntity(const QChar *&ch, const QString &textI
                 textOut += QChar(38);
             else if (entity == QLatin1String("quot"))
                 textOut += QChar(34);
+            else if (entity == QLatin1String("nbsp"))
+                textOut += QChar(QChar::Nbsp);
+            return;
+        } else if (*ch == QLatin1Char(' ')) {
+            QStringRef entity(&textIn, entityStart - 1, entityLength + 1);
+            textOut += entity + *ch;
             return;
         }
         ++entityLength;
@@ -654,10 +652,13 @@ bool QQuickStyledTextPrivate::parseAnchorAttributes(const QChar *&ch, const QStr
 void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QString &textIn, QString &textOut)
 {
     qreal imgWidth = 0.0;
+    QFontMetricsF fm(layout.font());
+    const qreal spaceWidth = fm.width(QChar::Nbsp);
+    const bool trailingSpace = textOut.endsWith(space);
 
     if (!updateImagePositions) {
         QQuickStyledTextImgTag *image = new QQuickStyledTextImgTag;
-        image->position = textOut.length() + 1;
+        image->position = textOut.length() + (trailingSpace ? 0 : 1);
 
         QPair<QStringRef,QStringRef> attr;
         do {
@@ -694,14 +695,16 @@ void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QStri
         }
 
         imgWidth = image->size.width();
+        image->offset = -std::fmod(imgWidth, spaceWidth) / 2.0;
         imgTags->append(image);
 
     } else {
         // if we already have a list of img tags for this text
         // we only want to update the positions of these tags.
         QQuickStyledTextImgTag *image = imgTags->value(nbImages);
-        image->position = textOut.length() + 1;
+        image->position = textOut.length() + (trailingSpace ? 0 : 1);
         imgWidth = image->size.width();
+        image->offset = -std::fmod(imgWidth, spaceWidth) / 2.0;
         QPair<QStringRef,QStringRef> attr;
         do {
             attr = parseAttribute(ch, textIn);
@@ -709,9 +712,9 @@ void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QStri
         nbImages++;
     }
 
-    QFontMetricsF fm(layout.font());
-    QString padding(qFloor(imgWidth / fm.width(QChar::Nbsp)), QChar::Nbsp);
-    textOut += QLatin1Char(' ');
+    QString padding(qFloor(imgWidth / spaceWidth), QChar::Nbsp);
+    if (!trailingSpace)
+        textOut += QLatin1Char(' ');
     textOut += padding;
     textOut += QLatin1Char(' ');
 }
