@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -40,6 +40,7 @@
 #include <QtQuick>
 #include <QtQuick/private/qquickrectangle_p.h>
 #include <QtQuick/private/qquickmousearea_p.h>
+#include <private/qv8engine_p.h>
 #include <qcolor.h>
 #include "../../shared/util.h"
 #include "testhttpserver.h"
@@ -103,17 +104,20 @@ private slots:
     void null();
     void loadEmptyUrl();
     void qmlCreateWindow();
-    void qmlCreateObject();
+    void qmlCreateObjectAutoParent_data();
+    void qmlCreateObjectAutoParent();
     void qmlCreateObjectWithProperties();
     void qmlIncubateObject();
     void qmlCreateParentReference();
     void async();
     void asyncHierarchy();
+    void asyncForceSync();
     void componentUrlCanonicalization();
     void onDestructionLookup();
     void onDestructionCount();
     void recursion();
     void recursionContinuation();
+    void callingContextForInitialProperties();
 
 private:
     QQmlEngine engine;
@@ -156,7 +160,7 @@ void tst_qqmlcomponent::qmlIncubateObject()
     QCOMPARE(object->property("test1").toBool(), true);
     QCOMPARE(object->property("test2").toBool(), false);
 
-    QTRY_VERIFY(object->property("test2").toBool() == true);
+    QTRY_VERIFY(object->property("test2").toBool());
 
     delete object;
 }
@@ -170,21 +174,73 @@ void tst_qqmlcomponent::qmlCreateWindow()
     QVERIFY(window);
 }
 
-void tst_qqmlcomponent::qmlCreateObject()
+void tst_qqmlcomponent::qmlCreateObjectAutoParent_data()
 {
+    QTest::addColumn<QString>("testFile");
+
+    QTest::newRow("createObject") << QStringLiteral("createObject.qml");
+    QTest::newRow("createQmlObject") <<  QStringLiteral("createQmlObject.qml");
+}
+
+
+void tst_qqmlcomponent::qmlCreateObjectAutoParent()
+{
+    QFETCH(QString, testFile);
+
     QQmlEngine engine;
-    QQmlComponent component(&engine, testFileUrl("createObject.qml"));
-    QObject *object = component.create();
-    QVERIFY(object != 0);
+    QQmlComponent component(&engine, testFileUrl(testFile));
+    QQuickItem *root = qobject_cast<QQuickItem *>(component.create());
+    QVERIFY(root);
+    QObject *qtobjectParent = root->property("qtobjectParent").value<QObject*>();
+    QQuickItem *itemParent = qobject_cast<QQuickItem *>(root->property("itemParent").value<QObject*>());
+    QQuickWindow *windowParent = qobject_cast<QQuickWindow *>(root->property("windowParent").value<QObject*>());
+    QVERIFY(qtobjectParent);
+    QVERIFY(itemParent);
+    QVERIFY(windowParent);
 
-    QObject *testObject1 = object->property("qobject").value<QObject*>();
-    QVERIFY(testObject1);
-    QVERIFY(testObject1->parent() == object);
+    QObject *qtobject_qtobject = root->property("qtobject_qtobject").value<QObject*>();
+    QObject *qtobject_item = root->property("qtobject_item").value<QObject*>();
+    QObject *qtobject_window = root->property("qtobject_window").value<QObject*>();
+    QObject *item_qtobject = root->property("item_qtobject").value<QObject*>();
+    QObject *item_item = root->property("item_item").value<QObject*>();
+    QObject *item_window = root->property("item_window").value<QObject*>();
+    QObject *window_qtobject = root->property("window_qtobject").value<QObject*>();
+    QObject *window_item = root->property("window_item").value<QObject*>();
+    QObject *window_window = root->property("window_window").value<QObject*>();
 
-    QObject *testObject2 = object->property("declarativeitem").value<QObject*>();
-    QVERIFY(testObject2);
-    QVERIFY(testObject2->parent() == object);
-    QCOMPARE(testObject2->metaObject()->className(), "QQuickItem");
+    QVERIFY(qtobject_qtobject);
+    QVERIFY(qtobject_item);
+    QVERIFY(qtobject_window);
+    QVERIFY(item_qtobject);
+    QVERIFY(item_item);
+    QVERIFY(item_window);
+    QVERIFY(window_qtobject);
+    QVERIFY(window_item);
+    QVERIFY(window_window);
+
+    QCOMPARE(qtobject_item->metaObject()->className(), "QQuickItem");
+    QCOMPARE(qtobject_window->metaObject()->className(), "QQuickWindow");
+    QCOMPARE(item_item->metaObject()->className(), "QQuickItem");
+    QCOMPARE(item_window->metaObject()->className(), "QQuickWindow");
+    QCOMPARE(window_item->metaObject()->className(), "QQuickItem");
+    QCOMPARE(window_window->metaObject()->className(), "QQuickWindow");
+
+    QCOMPARE(qtobject_qtobject->parent(), qtobjectParent);
+    QCOMPARE(qtobject_item->parent(), qtobjectParent);
+    QCOMPARE(qtobject_window->parent(), qtobjectParent);
+    QCOMPARE(item_qtobject->parent(), itemParent);
+    QCOMPARE(item_item->parent(), itemParent);
+    QCOMPARE(item_window->parent(), itemParent);
+    QCOMPARE(window_qtobject->parent(), windowParent);
+    QCOMPARE(window_item->parent(), windowParent);
+    QCOMPARE(window_window->parent(), windowParent);
+
+    QCOMPARE(qobject_cast<QQuickItem *>(qtobject_item)->parentItem(), (QQuickItem *)0);
+    QCOMPARE(qobject_cast<QQuickWindow *>(qtobject_window)->transientParent(), (QQuickWindow *)0);
+    QCOMPARE(qobject_cast<QQuickItem *>(item_item)->parentItem(), itemParent);
+    QCOMPARE(qobject_cast<QQuickWindow *>(item_window)->transientParent(), itemParent->window());
+    QCOMPARE(qobject_cast<QQuickItem *>(window_item)->parentItem(), windowParent->contentItem());
+    QCOMPARE(qobject_cast<QQuickWindow *>(window_window)->transientParent(), windowParent);
 }
 
 void tst_qqmlcomponent::qmlCreateObjectWithProperties()
@@ -197,7 +253,7 @@ void tst_qqmlcomponent::qmlCreateObjectWithProperties()
 
     QObject *testObject1 = object->property("declarativerectangle").value<QObject*>();
     QVERIFY(testObject1);
-    QVERIFY(testObject1->parent() == object);
+    QCOMPARE(testObject1->parent(), object);
     QCOMPARE(testObject1->property("x").value<int>(), 17);
     QCOMPARE(testObject1->property("y").value<int>(), 17);
     QCOMPARE(testObject1->property("color").value<QColor>(), QColor(255,255,255));
@@ -207,7 +263,7 @@ void tst_qqmlcomponent::qmlCreateObjectWithProperties()
 
     QObject *testObject2 = object->property("declarativeitem").value<QObject*>();
     QVERIFY(testObject2);
-    QVERIFY(testObject2->parent() == object);
+    QCOMPARE(testObject2->parent(), object);
     //QCOMPARE(testObject2->metaObject()->className(), "QDeclarativeItem_QML_2");
     QCOMPARE(testObject2->property("x").value<int>(), 17);
     QCOMPARE(testObject2->property("y").value<int>(), 17);
@@ -318,6 +374,35 @@ void tst_qqmlcomponent::asyncHierarchy()
     delete root;
 }
 
+void tst_qqmlcomponent::asyncForceSync()
+{
+    {
+        // 1) make sure that HTTP URLs cannot be completed synchronously
+        TestHTTPServer server;
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
+        server.serveDirectory(dataDirectory());
+
+        // ensure that the item hierarchy is compiled correctly.
+        QQmlComponent component(&engine);
+        component.loadUrl(server.url("/TestComponent.2.qml"), QQmlComponent::Asynchronous);
+        QCOMPARE(component.status(), QQmlComponent::Loading);
+        QQmlComponent component2(&engine, server.url("/TestComponent.2.qml"), QQmlComponent::PreferSynchronous);
+        QCOMPARE(component2.status(), QQmlComponent::Loading);
+    }
+    {
+        // 2) make sure that file:// URL can be completed synchronously
+
+        // ensure that the item hierarchy is compiled correctly.
+        QQmlComponent component(&engine);
+        component.loadUrl(testFileUrl("/TestComponent.2.qml"), QQmlComponent::Asynchronous);
+        QCOMPARE(component.status(), QQmlComponent::Loading);
+        QQmlComponent component2(&engine, testFileUrl("/TestComponent.2.qml"), QQmlComponent::PreferSynchronous);
+        QCOMPARE(component2.status(), QQmlComponent::Ready);
+        QCOMPARE(component.status(), QQmlComponent::Loading);
+        QTRY_COMPARE_WITH_TIMEOUT(component.status(), QQmlComponent::Ready, 0);
+    }
+}
+
 void tst_qqmlcomponent::componentUrlCanonicalization()
 {
     // ensure that url canonicalization succeeds so that type information
@@ -365,7 +450,7 @@ void tst_qqmlcomponent::componentUrlCanonicalization()
         QQmlComponent component(&engine, testFileUrl("componentUrlCanonicalization.5.qml"));
         QTest::ignoreMessage(QtWarningMsg, QLatin1String("QQmlComponent: Component is not ready").data());
         QScopedPointer<QObject> object(component.create());
-        QVERIFY(object == 0);
+        QVERIFY(object.isNull());
     }
 }
 
@@ -440,6 +525,63 @@ void tst_qqmlcomponent::recursionContinuation()
     QVERIFY(object->property("success").toBool());
 }
 
+class CallingContextCheckingClass : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(int value READ value WRITE setValue)
+public:
+    CallingContextCheckingClass()
+        : m_value(0)
+    {}
+
+    int value() const { return m_value; }
+    void setValue(int v) {
+        scopeObject.clear();
+        callingContextData.setContextData(0);
+
+        m_value = v;
+        QJSEngine *jsEngine = qjsEngine(this);
+        if (!jsEngine)
+            return;
+        QV4::ExecutionEngine *v4 = QV8Engine::getV4(jsEngine);
+        if (!v4)
+            return;
+        QV4::Scope scope(v4);
+        QV4::Scoped<QV4::QmlContext> qmlContext(scope, v4->qmlContext());
+        if (!qmlContext)
+            return;
+        callingContextData = qmlContext->qmlContext();
+        scopeObject = qmlContext->qmlScope();
+    }
+
+    int m_value;
+    QQmlGuardedContextData callingContextData;
+    QPointer<QObject> scopeObject;
+};
+
+void tst_qqmlcomponent::callingContextForInitialProperties()
+{
+    qmlRegisterType<CallingContextCheckingClass>("qqmlcomponenttest", 1, 0, "CallingContextCheckingClass");
+
+    QQmlComponent testFactory(&engine, testFileUrl("callingQmlContextComponent.qml"));
+
+    QQmlComponent component(&engine, testFileUrl("callingQmlContext.qml"));
+    QScopedPointer<QObject> root(component.beginCreate(engine.rootContext()));
+    QVERIFY(!root.isNull());
+    root->setProperty("factory", QVariant::fromValue(&testFactory));
+    component.completeCreate();
+    QTRY_VERIFY(qvariant_cast<QObject *>(root->property("incubatedObject")));
+    QObject *o = qvariant_cast<QObject *>(root->property("incubatedObject"));
+    CallingContextCheckingClass *checker = qobject_cast<CallingContextCheckingClass*>(o);
+    QVERIFY(checker);
+
+    QVERIFY(!checker->callingContextData.isNull());
+    QVERIFY(checker->callingContextData->urlString().endsWith(QStringLiteral("callingQmlContext.qml")));
+
+    QVERIFY(!checker->scopeObject.isNull());
+    QVERIFY(checker->scopeObject->metaObject()->indexOfProperty("incubatedObject") != -1);
+}
+
 QTEST_MAIN(tst_qqmlcomponent)
 
 #include "tst_qqmlcomponent.moc"
