@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -75,6 +75,8 @@ private slots:
     void urlInterceptor_data();
     void urlInterceptor();
 
+    void qmlContextProperties();
+
 public slots:
     QObject *createAQObjectForOwnershipTest ()
     {
@@ -90,7 +92,7 @@ void tst_qqmlengine::rootContext()
     QVERIFY(engine.rootContext());
 
     QCOMPARE(engine.rootContext()->engine(), &engine);
-    QVERIFY(engine.rootContext()->parentContext() == 0);
+    QVERIFY(!engine.rootContext()->parentContext());
 }
 
 class NetworkAccessManagerFactory : public QQmlNetworkAccessManagerFactory
@@ -119,8 +121,9 @@ void tst_qqmlengine::networkAccessManager()
     engine = new QQmlEngine;
     NetworkAccessManagerFactory factory;
     engine->setNetworkAccessManagerFactory(&factory);
-    QVERIFY(engine->networkAccessManagerFactory() == &factory);
-    QVERIFY(engine->networkAccessManager() == factory.manager);
+    QCOMPARE(engine->networkAccessManagerFactory(), &factory);
+    QNetworkAccessManager *engineNam = engine->networkAccessManager(); // calls NetworkAccessManagerFactory::create()
+    QCOMPARE(engineNam, factory.manager);
     delete engine;
 }
 
@@ -184,7 +187,7 @@ void tst_qqmlengine::baseUrl()
     dir.cdUp();
     QVERIFY(dir != QDir::current());
     QDir::setCurrent(dir.path());
-    QVERIFY(QDir::current() == dir);
+    QCOMPARE(QDir::current(), dir);
 
     QUrl cwd2 = QUrl::fromLocalFile(QDir::currentPath() + QDir::separator());
     QCOMPARE(engine.baseUrl(), cwd2);
@@ -200,11 +203,11 @@ void tst_qqmlengine::contextForObject()
     QQmlEngine *engine = new QQmlEngine;
 
     // Test null-object
-    QVERIFY(QQmlEngine::contextForObject(0) == 0);
+    QVERIFY(!QQmlEngine::contextForObject(0));
 
     // Test an object with no context
     QObject object;
-    QVERIFY(QQmlEngine::contextForObject(&object) == 0);
+    QVERIFY(!QQmlEngine::contextForObject(&object));
 
     // Test setting null-object
     QQmlEngine::setContextForObject(0, engine->rootContext());
@@ -214,18 +217,18 @@ void tst_qqmlengine::contextForObject()
 
     // Test setting context
     QQmlEngine::setContextForObject(&object, engine->rootContext());
-    QVERIFY(QQmlEngine::contextForObject(&object) == engine->rootContext());
+    QCOMPARE(QQmlEngine::contextForObject(&object), engine->rootContext());
 
     QQmlContext context(engine->rootContext());
 
     // Try changing context
     QTest::ignoreMessage(QtWarningMsg, "QQmlEngine::setContextForObject(): Object already has a QQmlContext");
     QQmlEngine::setContextForObject(&object, &context);
-    QVERIFY(QQmlEngine::contextForObject(&object) == engine->rootContext());
+    QCOMPARE(QQmlEngine::contextForObject(&object), engine->rootContext());
 
     // Delete context
     delete engine; engine = 0;
-    QVERIFY(QQmlEngine::contextForObject(&object) == 0);
+    QVERIFY(!QQmlEngine::contextForObject(&object));
 }
 
 void tst_qqmlengine::offlineStoragePath()
@@ -446,7 +449,7 @@ void tst_qqmlengine::failedCompilation()
     QQmlComponent component(&engine, testFileUrl(file));
     QVERIFY(!component.isReady());
     QScopedPointer<QObject> object(component.create());
-    QVERIFY(object == 0);
+    QVERIFY(object.isNull());
 
     engine.collectGarbage();
     engine.trimComponentCache();
@@ -470,7 +473,7 @@ void tst_qqmlengine::outputWarningsToStandardError()
     QQmlComponent c(&engine);
     c.setData("import QtQuick 2.0; QtObject { property int a: undefined }", QUrl());
 
-    QVERIFY(c.isReady() == true);
+    QVERIFY(c.isReady());
 
     QQmlTestMessageHandler messageHandler;
 
@@ -743,6 +746,15 @@ void tst_qqmlengine::urlInterceptor_data()
         << QStringLiteral("base file")
         << testFileUrl("interception/strings/intercepted/doesNotExist.file").toString()
         << QStringLiteral("file:///intercepted/doesNotExist.file");
+
+    QTest::newRow("InterceptIncludes")
+        << testFileUrl("interception/includes/urlInterceptor.qml")
+        << (QList<QQmlAbstractUrlInterceptor::DataType>() << QQmlAbstractUrlInterceptor::JavaScriptFile)
+        << testFileUrl("interception/includes/doesNotExist.file").toString()
+        << QStringLiteral("base file")
+        << QStringLiteral("intercepted include file")
+        << testFileUrl("interception/includes/doesNotExist.file").toString()
+        << QStringLiteral("file:///doesNotExist.file");
 }
 
 void tst_qqmlengine::urlInterceptor()
@@ -777,6 +789,18 @@ void tst_qqmlengine::urlInterceptor()
     QCOMPARE(o->property("absoluteUrl").toString(), expectedAbsoluteUrl);
 }
 
+void tst_qqmlengine::qmlContextProperties()
+{
+    QQmlEngine e;
+
+    QQmlComponent c(&e, testFileUrl("TypeofQmlProperty.qml"));
+    QObject *o = c.create();
+    if (!o) {
+        qDebug() << c.errorString();
+    }
+    QVERIFY(o);
+}
+
 QTEST_MAIN(tst_qqmlengine)
 
 #include "tst_qqmlengine.moc"
