@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -117,6 +109,7 @@ private slots:
     void stringLocaleCompare_data();
     void stringLocaleCompare();
 
+    void localeAsCppProperty();
 private:
     void addPropertyData(const QString &l);
     QVariant getProperty(QObject *obj, const QString &locale, const QString &property);
@@ -458,7 +451,7 @@ void tst_qqmllocale::firstDayOfWeek()
         Q_ARG(QVariant, QVariant(locale)));
 
     QVariant val = obj->property("firstDayOfWeek");
-    QVERIFY(val.type() == QVariant::Int);
+    QCOMPARE(val.type(), QVariant::Int);
 
     int day = int(QLocale(locale).firstDayOfWeek());
     if (day == 7) // JS Date days in range 0(Sunday) to 6(Saturday)
@@ -493,12 +486,12 @@ void tst_qqmllocale::weekDays()
         Q_ARG(QVariant, QVariant(locale)));
 
     QVariant val = obj->property("weekDays");
-    QVERIFY(val.type() == QVariant::List);
+    QCOMPARE(val.userType(), qMetaTypeId<QJSValue>());
 
     QList<QVariant> qmlDays = val.toList();
     QList<Qt::DayOfWeek> days = QLocale(locale).weekdays();
 
-    QVERIFY(days.count() == qmlDays.count());
+    QCOMPARE(days.count(), qmlDays.count());
 
     for (int i = 0; i < days.count(); ++i) {
         int day = int(days.at(i));
@@ -535,12 +528,12 @@ void tst_qqmllocale::uiLanguages()
         Q_ARG(QVariant, QVariant(locale)));
 
     QVariant val = obj->property("uiLanguages");
-    QVERIFY(val.type() == QVariant::List);
+    QCOMPARE(val.userType(), qMetaTypeId<QJSValue>());
 
     QList<QVariant> qmlLangs = val.toList();
     QStringList langs = QLocale(locale).uiLanguages();
 
-    QVERIFY(langs.count() == qmlLangs.count());
+    QCOMPARE(langs.count(), qmlLangs.count());
 
     for (int i = 0; i < langs.count(); ++i) {
         QCOMPARE(langs.at(i), qmlLangs.at(i).toString());
@@ -683,7 +676,7 @@ void tst_qqmllocale::addDateTimeFormatData(const QString &l)
     int i = 0;
     while (formats[i]) {
         QByteArray t(locale);
-        t += " ";
+        t += ' ';
         t += formats[i];
         QTest::newRow(t.constData()) << l << QString(formats[i]);
         ++i;
@@ -775,7 +768,7 @@ void tst_qqmllocale::addDateFormatData(const QString &l)
     int i = 0;
     while (formats[i]) {
         QByteArray t(locale);
-        t += " ";
+        t += ' ';
         t += formats[i];
         QTest::newRow(t.constData()) << l << QString(formats[i]);
         ++i;
@@ -867,7 +860,7 @@ void tst_qqmllocale::addTimeFormatData(const QString &l)
     int i = 0;
     while (formats[i]) {
         QByteArray t(locale);
-        t += " ";
+        t += ' ';
         t += formats[i];
         QTest::newRow(t.constData()) << l << QString(formats[i]);
         ++i;
@@ -931,7 +924,6 @@ void tst_qqmllocale::dateFromLocaleString_data()
 
 void tst_qqmllocale::dateFromLocaleString()
 {
-    QSKIP("Needs fixes in our date time parser");
     QFETCH(QString, locale);
     QFETCH(QString, format);
 
@@ -976,7 +968,6 @@ void tst_qqmllocale::dateFromLocaleDateString_data()
 
 void tst_qqmllocale::dateFromLocaleDateString()
 {
-    QSKIP("Needs fixes in our date time parser");
     QFETCH(QString, locale);
     QFETCH(QString, format);
 
@@ -1021,7 +1012,6 @@ void tst_qqmllocale::dateFromLocaleTimeString_data()
 
 void tst_qqmllocale::dateFromLocaleTimeString()
 {
-    QSKIP("Needs fixes in our date time parser");
     QFETCH(QString, locale);
     QFETCH(QString, format);
 
@@ -1223,6 +1213,41 @@ void tst_qqmllocale::stringLocaleCompare()
     QCOMPARE(obj->property("comparison").toInt(), QString::localeAwareCompare(string1, string2));
 }
 
+class Calendar : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(QLocale locale READ locale WRITE setLocale)
+public:
+    Calendar() {
+    }
+
+    QLocale locale() const {
+        return mLocale;
+    }
+
+    void setLocale(const QLocale &locale) {
+        mLocale = locale;
+    }
+private:
+    QLocale mLocale;
+};
+
+void tst_qqmllocale::localeAsCppProperty()
+{
+    QQmlComponent component(&engine);
+    qmlRegisterType<Calendar>("Test", 1, 0, "Calendar");
+    component.setData("import QtQml 2.2\nimport Test 1.0\nCalendar { locale: Qt.locale('en_GB'); property var testLocale }", QUrl());
+    QVERIFY(!component.isError());
+    QTRY_VERIFY(component.isReady());
+
+    Calendar *item = qobject_cast<Calendar*>(component.create());
+    QCOMPARE(item->property("locale").toLocale().name(), QLatin1String("en_GB"));
+
+    QVariant localeVariant(QLocale("nb_NO"));
+    item->setProperty("testLocale", localeVariant);
+    QCOMPARE(item->property("testLocale").toLocale().name(), QLatin1String("nb_NO"));
+}
+
 class DateFormatter : public QObject
 {
     Q_OBJECT
