@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -74,6 +74,8 @@ private slots:
     void send_ignoreData();
     void send_withdata();
     void send_withdata_data();
+    void send_options();
+    void send_options_data();
     void abort();
     void abort_unsent();
     void abort_opened();
@@ -85,6 +87,8 @@ private slots:
     void getAllResponseHeaders_unsent();
     void getAllResponseHeaders_sent();
     void getAllResponseHeaders_args();
+    void getBinaryData();
+    void getJsonData();
     void status();
     void status_data();
     void statusText();
@@ -97,6 +101,10 @@ private slots:
     void nonUtf8();
     void nonUtf8_data();
 
+    // WebDAV
+    void sendPropfind();
+    void sendPropfind_data();
+
     // Attributes
     void document();
     void element();
@@ -169,7 +177,7 @@ void tst_qqmlxmlhttprequest::callbackException()
     object->setProperty("which", which);
     component.completeCreate();
 
-    QTRY_VERIFY(object->property("threw").toBool() == true);
+    QTRY_VERIFY(object->property("threw").toBool());
 }
 
 // Test that the state value properties on the XMLHttpRequest constructor have the correct values.
@@ -255,7 +263,7 @@ void tst_qqmlxmlhttprequest::open()
     QCOMPARE(object->property("responseText").toBool(), true);
     QCOMPARE(object->property("responseXML").toBool(), true);
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 void tst_qqmlxmlhttprequest::open_data()
@@ -282,14 +290,55 @@ void tst_qqmlxmlhttprequest::open_invalid_method()
     QCOMPARE(object->property("exceptionThrown").toBool(), true);
 }
 
-// Test that calling XMLHttpRequest.open() with sync raises an exception
+class TestThreadedHTTPServer : public QObject
+{
+    Q_OBJECT
+public:
+    TestThreadedHTTPServer(const QUrl &expectUrl, const QUrl &replyUrl, const QUrl &bodyUrl)
+        : m_server(Q_NULLPTR) {
+        QMutexLocker locker(&m_lock);
+        moveToThread(&m_thread);
+        m_thread.start();
+        QMetaObject::invokeMethod(this, "start", Qt::QueuedConnection, Q_ARG(QUrl, expectUrl), Q_ARG(QUrl, replyUrl), Q_ARG(QUrl, bodyUrl));
+        m_startupCondition.wait(&m_lock);
+    }
+    ~TestThreadedHTTPServer() {
+        m_server->deleteLater();
+        m_thread.exit();
+        m_thread.wait();
+    }
+
+    QUrl serverBaseUrl;
+
+private slots:
+    void start(const QUrl &expectUrl, const QUrl &replyUrl, const QUrl &bodyUrl) {
+        QMutexLocker locker(&m_lock);
+        m_server = new TestHTTPServer;
+        QVERIFY2(m_server->listen(), qPrintable(m_server->errorString()));
+        serverBaseUrl = m_server->baseUrl();
+        QVERIFY(m_server->wait(expectUrl, replyUrl, bodyUrl));
+        m_startupCondition.wakeAll();
+    }
+
+private:
+    TestHTTPServer *m_server;
+    QThread m_thread;
+    QMutex m_lock;
+    QWaitCondition m_startupCondition;
+};
+
+// Test that calling XMLHttpRequest.open() with sync
 void tst_qqmlxmlhttprequest::open_sync()
 {
-    QQmlComponent component(&engine, testFileUrl("open_sync.qml"));
-    QScopedPointer<QObject> object(component.create());
-    QVERIFY(!object.isNull());
+    TestThreadedHTTPServer server(testFileUrl("open_network.expect"), testFileUrl("open_network.reply"), testFileUrl("testdocument.html"));
 
-    QCOMPARE(object->property("exceptionThrown").toBool(), true);
+    QQmlComponent component(&engine, testFileUrl("open_sync.qml"));
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QVERIFY(!object.isNull());
+    object->setProperty("url", server.serverBaseUrl.resolved(QStringLiteral("/testdocument.html")).toString());
+    component.completeCreate();
+
+    QCOMPARE(object->property("responseText").toString(), QStringLiteral("QML Rocks!\n"));
 }
 
 // Calling with incorrect arg count raises an exception
@@ -327,7 +376,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader()
     object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 // Test valid setRequestHeader() calls with different header cases
@@ -345,7 +394,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader_caseInsensitive()
     object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 // Test setting headers before open() throws exception
 void tst_qqmlxmlhttprequest::setRequestHeader_unsent()
@@ -410,7 +459,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader_illegalName()
     QCOMPARE(object->property("responseText").toBool(), true);
     QCOMPARE(object->property("responseXML").toBool(), true);
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 // Test that attempting to set a header after a request is sent throws an exception
@@ -430,7 +479,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader_sent()
 
     QCOMPARE(object->property("test").toBool(), true);
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 // Invalid arg count throws exception
@@ -461,7 +510,7 @@ void tst_qqmlxmlhttprequest::send_alreadySent()
     QVERIFY(!object.isNull());
 
     QCOMPARE(object->property("test").toBool(), true);
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 // Test that sends for GET, HEAD and DELETE ignore data
@@ -481,7 +530,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
         object->setProperty("url", server.urlString("/testdocument.html"));
         component.completeCreate();
 
-        QTRY_VERIFY(object->property("dataOK").toBool() == true);
+        QTRY_VERIFY(object->property("dataOK").toBool());
     }
 
     {
@@ -498,7 +547,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
         object->setProperty("url", server.urlString("/testdocument.html"));
         component.completeCreate();
 
-        QTRY_VERIFY(object->property("dataOK").toBool() == true);
+        QTRY_VERIFY(object->property("dataOK").toBool());
     }
 
     {
@@ -515,7 +564,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
         object->setProperty("url", server.urlString("/testdocument.html"));
         component.completeCreate();
 
-        QTRY_VERIFY(object->property("dataOK").toBool() == true);
+        QTRY_VERIFY(object->property("dataOK").toBool());
     }
 }
 
@@ -537,7 +586,7 @@ void tst_qqmlxmlhttprequest::send_withdata()
     object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 void tst_qqmlxmlhttprequest::send_withdata_data()
@@ -554,6 +603,47 @@ void tst_qqmlxmlhttprequest::send_withdata_data()
     QTest::newRow("Correct content-type - no charset") << "send_data.1.expect" << "send_data.7.qml";
 }
 
+void tst_qqmlxmlhttprequest::send_options()
+{
+    QFETCH(QString, url_suffix);
+    QFETCH(QString, file_expected);
+    QFETCH(QString, file_qml);
+    QFETCH(QString, file_reply);
+
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
+    QVERIFY(server.wait(testFileUrl(file_expected),
+                        testFileUrl(file_reply),
+                        testFileUrl("testdocument.html")));
+
+    QQmlComponent component(&engine, testFileUrl(file_qml));
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QVERIFY(!object.isNull());
+    QString url = server.baseUrl().toString();
+    if (url_suffix != "/")
+        url.append(QLatin1Char('/'));
+    if (!url_suffix.isEmpty())
+        url.append(url_suffix);
+    object->setProperty("url", url);
+    component.completeCreate();
+
+    QTRY_VERIFY(object->property("dataOK").toBool());
+    QTRY_VERIFY(object->property("headerOK").toBool());
+}
+
+void tst_qqmlxmlhttprequest::send_options_data()
+{
+    QTest::addColumn<QString>("url_suffix");
+    QTest::addColumn<QString>("file_expected");
+    QTest::addColumn<QString>("file_qml");
+    QTest::addColumn<QString>("file_reply");
+
+    QTest::newRow("OPTIONS (no data, no resource, no path)") << "" << "send_data.8.expect" << "send_data.8.qml" << "send_data.2.reply";
+    QTest::newRow("OPTIONS (no data, no resource, path- \"/\")") << "/" << "send_data.8.expect" << "send_data.8.qml" << "send_data.2.reply";
+    QTest::newRow("OPTIONS (no data, with resource)") << "testdocument.html" << "send_data.9.expect" << "send_data.8.qml" << "send_data.2.reply";
+    QTest::newRow("OPTIONS (with data)") << "testdocument.html" << "send_data.10.expect" << "send_data.9.qml" << "send_data.2.reply";
+}
+
 // Test abort() has no effect in unsent state
 void tst_qqmlxmlhttprequest::abort_unsent()
 {
@@ -570,7 +660,7 @@ void tst_qqmlxmlhttprequest::abort_unsent()
     QCOMPARE(object->property("responseText").toBool(), true);
     QCOMPARE(object->property("responseXML").toBool(), true);
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 // Test abort() cancels an open (but unsent) request
@@ -589,7 +679,7 @@ void tst_qqmlxmlhttprequest::abort_opened()
     QCOMPARE(object->property("responseText").toBool(), true);
     QCOMPARE(object->property("responseXML").toBool(), true);
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 // Test abort() aborts in progress send
@@ -615,7 +705,7 @@ void tst_qqmlxmlhttprequest::abort()
     QCOMPARE(object->property("didNotSeeUnsent").toBool(), true);
     QCOMPARE(object->property("endStateUnsent").toBool(), true);
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 }
 
 void tst_qqmlxmlhttprequest::getResponseHeader()
@@ -640,7 +730,7 @@ void tst_qqmlxmlhttprequest::getResponseHeader()
     QCOMPARE(object->property("readyState").toBool(), true);
     QCOMPARE(object->property("openedState").toBool(), true);
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("headersReceivedState").toBool(), true);
     QCOMPARE(object->property("headersReceivedNullHeader").toBool(), true);
@@ -682,7 +772,7 @@ void tst_qqmlxmlhttprequest::getResponseHeader_args()
     QScopedPointer<QObject> object(component.create());
     QVERIFY(!object.isNull());
 
-    QTRY_VERIFY(object->property("exceptionThrown").toBool() == true);
+    QTRY_VERIFY(object->property("exceptionThrown").toBool());
 }
 
 void tst_qqmlxmlhttprequest::getAllResponseHeaders()
@@ -706,7 +796,7 @@ void tst_qqmlxmlhttprequest::getAllResponseHeaders()
     QCOMPARE(object->property("readyState").toBool(), true);
     QCOMPARE(object->property("openedState").toBool(), true);
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("headersReceivedState").toBool(), true);
     QCOMPARE(object->property("headersReceivedHeader").toBool(), true);
@@ -742,7 +832,43 @@ void tst_qqmlxmlhttprequest::getAllResponseHeaders_args()
     QScopedPointer<QObject> object(component.create());
     QVERIFY(!object.isNull());
 
-    QTRY_VERIFY(object->property("exceptionThrown").toBool() == true);
+    QTRY_VERIFY(object->property("exceptionThrown").toBool());
+}
+
+void tst_qqmlxmlhttprequest::getBinaryData()
+{
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
+    QVERIFY(server.wait(testFileUrl("receive_binary_data.expect"),
+                        testFileUrl("receive_binary_data.reply"),
+                        testFileUrl("qml_logo.png")));
+
+    QQmlComponent component(&engine, testFileUrl("receiveBinaryData.qml"));
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QVERIFY(!object.isNull());
+    object->setProperty("url", server.urlString("/gml_logo.png"));
+    component.completeCreate();
+
+    QFileInfo fileInfo("data/qml_logo.png");
+    QTRY_COMPARE(object->property("readSize").toInt(), fileInfo.size());
+    QCOMPARE(object->property("status").toInt(), 200);
+}
+
+void tst_qqmlxmlhttprequest::getJsonData()
+{
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
+    QVERIFY(server.wait(testFileUrl("receive_json_data.expect"),
+                        testFileUrl("receive_binary_data.reply"),
+                        testFileUrl("json.data")));
+
+    QQmlComponent component(&engine, testFileUrl("receiveJsonData.qml"));
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QVERIFY(!object.isNull());
+    object->setProperty("url", server.urlString("/json.data"));
+    component.completeCreate();
+
+    QTRY_VERIFY(object->property("result").toBool());
 }
 
 void tst_qqmlxmlhttprequest::status()
@@ -763,7 +889,7 @@ void tst_qqmlxmlhttprequest::status()
     object->setProperty("expectedStatus", status);
     component.completeCreate();
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("unsentException").toBool(), true);
     QCOMPARE(object->property("openedException").toBool(), true);
@@ -802,7 +928,7 @@ void tst_qqmlxmlhttprequest::statusText()
     object->setProperty("expectedStatus", statusText);
     component.completeCreate();
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("unsentException").toBool(), true);
     QCOMPARE(object->property("openedException").toBool(), true);
@@ -842,7 +968,7 @@ void tst_qqmlxmlhttprequest::responseText()
     object->setProperty("expectedText", responseText);
     component.completeCreate();
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("unsent").toBool(), true);
     QCOMPARE(object->property("opened").toBool(), true);
@@ -863,6 +989,7 @@ void tst_qqmlxmlhttprequest::responseText_data()
     QTest::newRow("empty body") << testFileUrl("status.200.reply") << QUrl() << "";
     QTest::newRow("Not Found") << testFileUrl("status.404.reply") << testFileUrl("testdocument.html") << "QML Rocks!\n";
     QTest::newRow("Bad Request") << testFileUrl("status.400.reply") << testFileUrl("testdocument.html") << "QML Rocks!\n";
+    QTest::newRow("Internal server error") << testFileUrl("status.500.reply") << testFileUrl("testdocument.html") << "QML Rocks!\n";
 }
 
 void tst_qqmlxmlhttprequest::nonUtf8()
@@ -878,7 +1005,7 @@ void tst_qqmlxmlhttprequest::nonUtf8()
     object->setProperty("fileName", fileName);
     QMetaObject::invokeMethod(object.data(), "startRequest");
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("responseText").toString(), responseText);
 
@@ -904,6 +1031,46 @@ void tst_qqmlxmlhttprequest::nonUtf8_data()
     QTest::newRow("responseXML") << "utf16.xml" << "<?xml version=\"1.0\" encoding=\"UTF-16\" standalone='yes'?>\n<root>\n" + uc + "\n</root>\n" << QString('\n' + uc + '\n');
 }
 
+void tst_qqmlxmlhttprequest::sendPropfind()
+{
+    const QString prefix = "WebDAV//";
+
+    QFETCH(QString, qml);
+    QFETCH(QString, resource);
+    QFETCH(QString, expectedFile);
+    QFETCH(QString, replyHeader);
+    QFETCH(QString, replyBody);
+
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
+
+    QVERIFY(server.wait(testFileUrl(prefix + expectedFile),
+                testFileUrl(prefix + replyHeader),
+                testFileUrl(prefix + replyBody)));
+
+    QQmlComponent component(&engine, testFileUrl(prefix + qml));
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QVERIFY(!object.isNull());
+    object->setProperty("url", server.urlString(resource));
+    component.completeCreate();
+
+    QTRY_VERIFY(object->property("xmlTest").toBool());
+    QCOMPARE(object->property("typeTest").toBool(), true);
+}
+
+void tst_qqmlxmlhttprequest::sendPropfind_data()
+{
+    QTest::addColumn<QString>("qml");
+    QTest::addColumn<QString>("resource");
+    QTest::addColumn<QString>("expectedFile");
+    QTest::addColumn<QString>("replyHeader");
+    QTest::addColumn<QString>("replyBody");
+
+    QTest::newRow("Send PROPFIND for file (bigbox, author, DingALing, Random properties). Get response with responseXML.") << "sendPropfind.responseXML.qml" << "/file" << "propfind.file.expect" << "propfind.file.reply.header" << "propfind.file.reply.body";
+    QTest::newRow("Send PROPFIND for file (bigbox, author, DingALing, Random properties). Get response with response.") << "sendPropfind.response.qml" << "/file" << "propfind.file.expect" << "propfind.file.reply.header" << "propfind.file.reply.body";
+    QTest::newRow("Send PROPFIND \"allprop\" request for collection.") << "sendPropfind.collection.allprop.qml" << "/container/" << "propfind.collection.allprop.expect" << "propfind.file.reply.header" << "propfind.collection.allprop.reply.body";
+}
+
 // Test that calling hte XMLHttpRequest methods on a non-XMLHttpRequest object
 // throws an exception
 void tst_qqmlxmlhttprequest::invalidMethodUsage()
@@ -942,7 +1109,7 @@ void tst_qqmlxmlhttprequest::redirects()
         object->setProperty("expectedText", "");
         component.completeCreate();
 
-        QTRY_VERIFY(object->property("done").toBool() == true);
+        QTRY_VERIFY(object->property("done").toBool());
         QCOMPARE(object->property("dataOK").toBool(), true);
     }
 
@@ -959,7 +1126,7 @@ void tst_qqmlxmlhttprequest::redirects()
         object->setProperty("expectedText", "");
         component.completeCreate();
 
-        QTRY_VERIFY(object->property("done").toBool() == true);
+        QTRY_VERIFY(object->property("done").toBool());
         QCOMPARE(object->property("dataOK").toBool(), true);
     }
 
@@ -980,7 +1147,7 @@ void tst_qqmlxmlhttprequest::redirects()
             if (object->property("done").toBool()) break;
             QTest::qWait(50);
         }
-        QVERIFY(object->property("done").toBool() == true);
+        QVERIFY(object->property("done").toBool());
 
         QCOMPARE(object->property("dataOK").toBool(), true);
     }
@@ -992,7 +1159,7 @@ void tst_qqmlxmlhttprequest::responseXML_invalid()
     QScopedPointer<QObject> object(component.create());
     QVERIFY(!object.isNull());
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("xmlNull").toBool(), true);
 }
@@ -1004,7 +1171,7 @@ void tst_qqmlxmlhttprequest::document()
     QScopedPointer<QObject> object(component.create());
     QVERIFY(!object.isNull());
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("xmlTest").toBool(), true);
 }
@@ -1016,7 +1183,7 @@ void tst_qqmlxmlhttprequest::element()
     QScopedPointer<QObject> object(component.create());
     QVERIFY(!object.isNull());
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("xmlTest").toBool(), true);
 }
@@ -1028,7 +1195,7 @@ void tst_qqmlxmlhttprequest::attr()
     QScopedPointer<QObject> object(component.create());
     QVERIFY(!object.isNull());
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("xmlTest").toBool(), true);
 }
@@ -1040,9 +1207,10 @@ void tst_qqmlxmlhttprequest::text()
     QScopedPointer<QObject> object(component.create());
     QVERIFY(!object.isNull());
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("xmlTest").toBool(), true);
+    QCOMPARE(object->property("status").toInt(), 200);
 }
 
 // Test the CDataSection DOM element
@@ -1052,9 +1220,10 @@ void tst_qqmlxmlhttprequest::cdata()
     QScopedPointer<QObject> object(component.create());
     QVERIFY(!object.isNull());
 
-    QTRY_VERIFY(object->property("dataOK").toBool() == true);
+    QTRY_VERIFY(object->property("dataOK").toBool());
 
     QCOMPARE(object->property("xmlTest").toBool(), true);
+    QCOMPARE(object->property("status").toInt(), 200);
 }
 
 void tst_qqmlxmlhttprequest::stateChangeCallingContext()
@@ -1076,7 +1245,7 @@ void tst_qqmlxmlhttprequest::stateChangeCallingContext()
     object->setProperty("serverBaseUrl", server.baseUrl().toString());
     component.completeCreate();
     server.sendDelayedItem();
-    QTRY_VERIFY(object->property("success").toBool() == true);
+    QTRY_VERIFY(object->property("success").toBool());
 }
 
 QTEST_MAIN(tst_qqmlxmlhttprequest)
