@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -84,7 +76,9 @@ private slots:
     void position_external();
     void drop_internal();
 //    void drop_external();
+    void competingDrags();
     void simultaneousDrags();
+    void dropStuff();
 
 private:
     QQmlEngine engine;
@@ -767,6 +761,50 @@ void tst_QQuickDropArea::drop_internal()
 //{
 //}
 
+void tst_QQuickDropArea::competingDrags()
+{
+    QQuickWindow window;
+    QQmlComponent component(&engine);
+    component.setData(
+            "import QtQuick 2.0\n"
+            "DropArea {\n"
+                "width: 100; height: 100\n"
+                "objectName: \"dropArea1\"\n"
+                "property string statuslol\n"
+                "onEntered: { statuslol = 'parent' }\n"
+                "DropArea {\n"
+                    "objectName: \"dropArea2\"\n"
+                    "width: 100; height: 100\n"
+                    "property bool acceptsEnters: true\n"
+                    "onEntered: { parent.statuslol = 'son'; drag.accepted = acceptsEnters; }\n"
+                "}\n"
+                "Item {\n"
+                    "objectName: \"dragItem\"\n"
+                    "x: 50; y: 50\n"
+                    "width: 10; height: 10\n"
+                "}\n"
+            "}\n", QUrl());
+
+    QScopedPointer<QObject> object(component.create());
+    QQuickItem *dropArea1 = qobject_cast<QQuickItem *>(object.data());
+    QVERIFY(dropArea1);
+    dropArea1->setParentItem(window.contentItem());
+
+    QQuickItem *dropArea2 = dropArea1->findChild<QQuickItem *>("dropArea2");
+    QVERIFY(dropArea2);
+
+    QQuickItem *dragItem = dropArea1->findChild<QQuickItem *>("dragItem");
+    QVERIFY(dragItem);
+
+    QCOMPARE(evaluate<QString>(dropArea1, "statuslol"), QStringLiteral(""));
+    evaluate<void>(dragItem, "Drag.active = true");
+    QCOMPARE(evaluate<QString>(dropArea1, "statuslol"), QStringLiteral("son"));
+    evaluate<void>(dragItem, "Drag.active = false");
+    evaluate<void>(dropArea2, "acceptsEnters = false");
+    evaluate<void>(dragItem, "Drag.active = true");
+    QCOMPARE(evaluate<QString>(dropArea1, "statuslol"), QStringLiteral("parent"));
+}
+
 void tst_QQuickDropArea::simultaneousDrags()
 {
     QQuickWindow window;
@@ -777,6 +815,7 @@ void tst_QQuickDropArea::simultaneousDrags()
                 "property int enterEvents: 0\n"
                 "property int exitEvents: 0\n"
                 "width: 100; height: 100\n"
+                "objectName: \"dropArea1\"\n"
                 "keys: [\"red\", \"text/x-red\"]\n"
                 "onEntered: {++enterEvents}\n"
                 "onExited: {++exitEvents}\n"
@@ -827,8 +866,8 @@ void tst_QQuickDropArea::simultaneousDrags()
     evaluate<void>(dropArea1, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dropArea2, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dragItem1, "Drag.active = true");
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 1);
@@ -837,32 +876,33 @@ void tst_QQuickDropArea::simultaneousDrags()
     evaluate<void>(dropArea1, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dropArea2, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dragItem2, "Drag.active = true");
+    //DropArea discards events if already contains something being dragged in
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     evaluate<void>(dragItem2, "Drag.active = false");
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     evaluate<void>(dragItem2, "Drag.active = true");
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 2);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     evaluate<void>(dragItem1, "Drag.active = false");
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 2);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
@@ -873,7 +913,7 @@ void tst_QQuickDropArea::simultaneousDrags()
     evaluate<void>(dragItem2, "Drag.active = false");
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
@@ -882,8 +922,8 @@ void tst_QQuickDropArea::simultaneousDrags()
     evaluate<void>(dropArea1, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dropArea2, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dragItem1, "Drag.active = true");
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 1);
@@ -892,32 +932,33 @@ void tst_QQuickDropArea::simultaneousDrags()
     evaluate<void>(dropArea1, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dropArea2, "{ enterEvents = 0; exitEvents = 0 }");
     QWindowSystemInterface::handleDrag(&window, &data, QPoint(50, 50), Qt::CopyAction);
+    //Same as in the first case, dropArea2 already contains a drag, dropArea1 will get the event
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     QWindowSystemInterface::handleDrag(&alternateWindow, &data, QPoint(50, 50), Qt::CopyAction);
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     QWindowSystemInterface::handleDrag(&window, &data, QPoint(50, 50), Qt::CopyAction);
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 2);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     evaluate<void>(dragItem1, "Drag.active = false");
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 2);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
@@ -928,7 +969,7 @@ void tst_QQuickDropArea::simultaneousDrags()
     QWindowSystemInterface::handleDrag(&alternateWindow, &data, QPoint(50, 50), Qt::CopyAction);
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
@@ -937,8 +978,8 @@ void tst_QQuickDropArea::simultaneousDrags()
     evaluate<void>(dropArea1, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dropArea2, "{ enterEvents = 0; exitEvents = 0 }");
     QWindowSystemInterface::handleDrag(&window, &data, QPoint(50, 50), Qt::CopyAction);
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 1);
@@ -948,31 +989,31 @@ void tst_QQuickDropArea::simultaneousDrags()
     evaluate<void>(dropArea2, "{ enterEvents = 0; exitEvents = 0 }");
     evaluate<void>(dragItem2, "Drag.active = true");
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     evaluate<void>(dragItem2, "Drag.active = false");
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 1);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     evaluate<void>(dragItem2, "Drag.active = true");
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 2);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), true);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
 
     QWindowSystemInterface::handleDrag(&alternateWindow, &data, QPoint(50, 50), Qt::CopyAction);
-    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
-    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
+    QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), true);
+    QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 2);
     QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
@@ -983,7 +1024,7 @@ void tst_QQuickDropArea::simultaneousDrags()
     evaluate<void>(dragItem2, "Drag.active = false");
     QCOMPARE(evaluate<bool>(dropArea1, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea1, "enterEvents"), 0);
-    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 0);
+    QCOMPARE(evaluate<int>(dropArea1, "exitEvents"), 1);
     QCOMPARE(evaluate<bool>(dropArea2, "containsDrag"), false);
     QCOMPARE(evaluate<int>(dropArea2, "enterEvents"), 0);
     QCOMPARE(evaluate<int>(dropArea2, "exitEvents"), 0);
@@ -1117,6 +1158,34 @@ void tst_QQuickDropArea::simultaneousDrags()
     QWindowSystemInterface::handleDrop(&alternateWindow, &data, QPoint(50, 50), Qt::CopyAction);
 }
 
+void tst_QQuickDropArea::dropStuff()
+{
+    QQuickWindow window;
+    QQmlComponent component(&engine);
+    component.setData(
+            "import QtQuick 2.3\n"
+            "DropArea {\n"
+                "width: 100; height: 100\n"
+                "property var array\n"
+                "onDropped: { array = drop.getDataAsArrayBuffer('text/x-red'); }\n"
+            "}", QUrl());
+
+    QScopedPointer<QObject> object(component.create());
+    QQuickItem *dropArea = qobject_cast<QQuickItem *>(object.data());
+    QVERIFY(dropArea);
+    dropArea->setParentItem(window.contentItem());
+
+    QMimeData data;
+    data.setData("text/x-red", "red");
+
+    QCOMPARE(evaluate<QVariant>(dropArea, "array"), QVariant());
+
+    QWindowSystemInterface::handleDrag(&window, &data, QPoint(50, 50), Qt::CopyAction);
+    QWindowSystemInterface::handleDrop(&window, &data, QPoint(50, 50), Qt::CopyAction);
+    QCOMPARE(evaluate<int>(dropArea, "array.byteLength"), 3);
+    QCOMPARE(evaluate<QByteArray>(dropArea, "array"), QByteArray("red"));
+}
+
 QTEST_MAIN(tst_QQuickDropArea)
 
 #include "tst_qquickdroparea.moc"
