@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -86,6 +78,8 @@ private slots:
     void resetDrag();
     void dragging_data() { acceptedButton_data(); }
     void dragging();
+    void dragSmoothed();
+    void dragThreshold_data();
     void dragThreshold();
     void invalidDrag_data() { rejectedButton_data(); }
     void invalidDrag();
@@ -95,6 +89,7 @@ private slots:
     void updateMouseAreaPosOnResize();
     void noOnClickedWithPressAndHold();
     void onMousePressRejected();
+    void pressedCanceledOnWindowDeactivate_data();
     void pressedCanceledOnWindowDeactivate();
     void doubleClick_data() { acceptedButton_data(); }
     void doubleClick();
@@ -122,6 +117,8 @@ private slots:
     void moveAndReleaseWithoutPress();
     void nestedStopAtBounds();
     void nestedStopAtBounds_data();
+    void containsPress_data();
+    void containsPress();
 
 private:
     void acceptedButton_data();
@@ -173,7 +170,7 @@ void tst_QQuickMouseArea::dragProperties()
     // target
     QQuickItem *blackRect = window.rootObject()->findChild<QQuickItem*>("blackrect");
     QVERIFY(blackRect != 0);
-    QVERIFY(blackRect == drag->target());
+    QCOMPARE(blackRect, drag->target());
     QQuickItem *rootItem = qobject_cast<QQuickItem*>(window.rootObject());
     QVERIFY(rootItem != 0);
     QSignalSpy targetSpy(drag, SIGNAL(targetChanged()));
@@ -271,14 +268,14 @@ void tst_QQuickMouseArea::resetDrag()
     // target
     QQuickItem *blackRect = window.rootObject()->findChild<QQuickItem*>("blackrect");
     QVERIFY(blackRect != 0);
-    QVERIFY(blackRect == drag->target());
+    QCOMPARE(blackRect, drag->target());
     QQuickItem *rootItem = qobject_cast<QQuickItem*>(window.rootObject());
     QVERIFY(rootItem != 0);
     QSignalSpy targetSpy(drag, SIGNAL(targetChanged()));
     QVERIFY(drag->target() != 0);
     window.rootContext()->setContextProperty("haveTarget", QVariant(false));
     QCOMPARE(targetSpy.count(),1);
-    QVERIFY(drag->target() == 0);
+    QVERIFY(!drag->target());
 }
 
 void tst_QQuickMouseArea::dragging()
@@ -304,7 +301,7 @@ void tst_QQuickMouseArea::dragging()
     // target
     QQuickItem *blackRect = window.rootObject()->findChild<QQuickItem*>("blackrect");
     QVERIFY(blackRect != 0);
-    QVERIFY(blackRect == drag->target());
+    QCOMPARE(blackRect, drag->target());
 
     QVERIFY(!drag->active());
 
@@ -334,8 +331,7 @@ void tst_QQuickMouseArea::dragging()
     QCOMPARE(blackRect->y(), 61.0);
 }
 
-
-void tst_QQuickMouseArea::dragThreshold()
+void tst_QQuickMouseArea::dragSmoothed()
 {
     QQuickView window;
     QByteArray errorMessage;
@@ -347,13 +343,68 @@ void tst_QQuickMouseArea::dragThreshold()
 
     QQuickMouseArea *mouseRegion = window.rootObject()->findChild<QQuickMouseArea*>("mouseregion");
     QQuickDrag *drag = mouseRegion->drag();
+    drag->setThreshold(5);
+
+    mouseRegion->setAcceptedButtons(Qt::LeftButton);
+    QQuickItem *blackRect = window.rootObject()->findChild<QQuickItem*>("blackrect");
+    QVERIFY(blackRect != 0);
+    QCOMPARE(blackRect, drag->target());
+    QVERIFY(!drag->active());
+    QTest::mousePress(&window, Qt::LeftButton, 0, QPoint(100,100));
+    QVERIFY(!drag->active());
+    QTest::mouseMove(&window, QPoint(100, 102), 50);
+    QTest::mouseMove(&window, QPoint(100, 106), 50);
+    QTest::mouseMove(&window, QPoint(100, 122), 50);
+    QTRY_COMPARE(blackRect->x(), 50.0);
+    QTRY_COMPARE(blackRect->y(), 66.0);
+    QTest::mouseRelease(&window, Qt::LeftButton, 0, QPoint(100,122));
+
+    // reset rect position
+    blackRect->setX(50.0);
+    blackRect->setY(50.0);
+
+    // now try with smoothed disabled
+    drag->setSmoothed(false);
+    QVERIFY(!drag->active());
+    QTest::mousePress(&window, Qt::LeftButton, 0, QPoint(100,100));
+    QVERIFY(!drag->active());
+    QTest::mouseMove(&window, QPoint(100, 102), 50);
+    QTest::mouseMove(&window, QPoint(100, 106), 50);
+    QTest::mouseMove(&window, QPoint(100, 122), 50);
+    QTRY_COMPARE(blackRect->x(), 50.0);
+    QTRY_COMPARE(blackRect->y(), 72.0);
+    QTest::mouseRelease(&window, Qt::LeftButton, 0, QPoint(100, 122));
+}
+
+void tst_QQuickMouseArea::dragThreshold_data()
+{
+    QTest::addColumn<bool>("preventStealing");
+    QTest::newRow("without preventStealing") << false;
+    QTest::newRow("with preventStealing") << true;
+}
+
+void tst_QQuickMouseArea::dragThreshold()
+{
+    QFETCH(bool, preventStealing);
+
+    QQuickView window;
+    QByteArray errorMessage;
+    QVERIFY2(initView(window, testFileUrl("dragging.qml"), true, &errorMessage), errorMessage.constData());
+
+    window.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+    QVERIFY(window.rootObject() != 0);
+
+    QQuickMouseArea *mouseRegion = window.rootObject()->findChild<QQuickMouseArea*>("mouseregion");
+    mouseRegion->setPreventStealing(preventStealing);
+    QQuickDrag *drag = mouseRegion->drag();
 
     drag->setThreshold(5);
 
     mouseRegion->setAcceptedButtons(Qt::LeftButton);
     QQuickItem *blackRect = window.rootObject()->findChild<QQuickItem*>("blackrect");
     QVERIFY(blackRect != 0);
-    QVERIFY(blackRect == drag->target());
+    QCOMPARE(blackRect, drag->target());
     QVERIFY(!drag->active());
     QTest::mousePress(&window, Qt::LeftButton, 0, QPoint(100,100));
     QVERIFY(!drag->active());
@@ -411,7 +462,7 @@ void tst_QQuickMouseArea::invalidDrag()
     // target
     QQuickItem *blackRect = window.rootObject()->findChild<QQuickItem*>("blackrect");
     QVERIFY(blackRect != 0);
-    QVERIFY(blackRect == drag->target());
+    QCOMPARE(blackRect, drag->target());
 
     QVERIFY(!drag->active());
 
@@ -461,7 +512,7 @@ void tst_QQuickMouseArea::cancelDragging()
     // target
     QQuickItem *blackRect = window.rootObject()->findChild<QQuickItem*>("blackrect");
     QVERIFY(blackRect != 0);
-    QVERIFY(blackRect == drag->target());
+    QCOMPARE(blackRect, drag->target());
 
     QVERIFY(!drag->active());
 
@@ -615,7 +666,7 @@ void tst_QQuickMouseArea::noOnClickedWithPressAndHold()
         QMouseEvent pressEvent(QEvent::MouseButtonPress, QPoint(100, 100), Qt::LeftButton, Qt::LeftButton, 0);
         QGuiApplication::sendEvent(&window, &pressEvent);
 
-        QVERIFY(mouseArea->pressedButtons() == Qt::LeftButton);
+        QCOMPARE(mouseArea->pressedButtons(), Qt::LeftButton);
         QVERIFY(!window.rootObject()->property("clicked").toBool());
         QVERIFY(!window.rootObject()->property("held").toBool());
 
@@ -692,8 +743,19 @@ void tst_QQuickMouseArea::onMousePressRejected()
     QVERIFY(!window.rootObject()->property("mr1_canceled").toBool());
     QVERIFY(!window.rootObject()->property("mr2_released").toBool());
 }
+
+void tst_QQuickMouseArea::pressedCanceledOnWindowDeactivate_data()
+{
+    QTest::addColumn<bool>("doubleClick");
+    QTest::newRow("simple click") << false;
+    QTest::newRow("double click") << true;
+}
+
+
 void tst_QQuickMouseArea::pressedCanceledOnWindowDeactivate()
 {
+    QFETCH(bool, doubleClick);
+
     QQuickView window;
     QByteArray errorMessage;
     QVERIFY2(initView(window, testFileUrl("pressedCanceled.qml"), true, &errorMessage), errorMessage.constData());
@@ -702,14 +764,41 @@ void tst_QQuickMouseArea::pressedCanceledOnWindowDeactivate()
     QVERIFY(window.rootObject() != 0);
     QVERIFY(!window.rootObject()->property("pressed").toBool());
     QVERIFY(!window.rootObject()->property("canceled").toBool());
-    QVERIFY(!window.rootObject()->property("released").toBool());
+
+    int expectedRelease = 0;
+    int expectedClicks = 0;
+    QCOMPARE(window.rootObject()->property("released").toInt(), expectedRelease);
+    QCOMPARE(window.rootObject()->property("clicked").toInt(), expectedClicks);
+
 
     QMouseEvent pressEvent(QEvent::MouseButtonPress, QPoint(100, 100), Qt::LeftButton, Qt::LeftButton, 0);
+    QMouseEvent releaseEvent(QEvent::MouseButtonRelease, QPoint(100, 100), Qt::LeftButton, Qt::LeftButton, 0);
+
     QGuiApplication::sendEvent(&window, &pressEvent);
 
     QVERIFY(window.rootObject()->property("pressed").toBool());
     QVERIFY(!window.rootObject()->property("canceled").toBool());
-    QVERIFY(!window.rootObject()->property("released").toBool());
+    QCOMPARE(window.rootObject()->property("released").toInt(), expectedRelease);
+    QCOMPARE(window.rootObject()->property("clicked").toInt(), expectedClicks);
+
+    if (doubleClick) {
+        QGuiApplication::sendEvent(&window, &releaseEvent);
+        QVERIFY(!window.rootObject()->property("pressed").toBool());
+        QVERIFY(!window.rootObject()->property("canceled").toBool());
+        QCOMPARE(window.rootObject()->property("released").toInt(), ++expectedRelease);
+        QCOMPARE(window.rootObject()->property("clicked").toInt(), ++expectedClicks);
+
+        QGuiApplication::sendEvent(&window, &pressEvent);
+        QMouseEvent pressEvent2(QEvent::MouseButtonDblClick, QPoint(100, 100), Qt::LeftButton, Qt::LeftButton, 0);
+        QGuiApplication::sendEvent(&window, &pressEvent2);
+
+        QVERIFY(window.rootObject()->property("pressed").toBool());
+        QVERIFY(!window.rootObject()->property("canceled").toBool());
+        QCOMPARE(window.rootObject()->property("released").toInt(), expectedRelease);
+        QCOMPARE(window.rootObject()->property("clicked").toInt(), expectedClicks);
+        QCOMPARE(window.rootObject()->property("doubleClicked").toInt(), 1);
+    }
+
 
     QWindow *secondWindow = qvariant_cast<QWindow*>(window.rootObject()->property("secondWindow"));
     secondWindow->setProperty("visible", true);
@@ -717,22 +806,24 @@ void tst_QQuickMouseArea::pressedCanceledOnWindowDeactivate()
 
     QVERIFY(!window.rootObject()->property("pressed").toBool());
     QVERIFY(window.rootObject()->property("canceled").toBool());
-    QVERIFY(!window.rootObject()->property("released").toBool());
+    QCOMPARE(window.rootObject()->property("released").toInt(), expectedRelease);
+    QCOMPARE(window.rootObject()->property("clicked").toInt(), expectedClicks);
 
     //press again
     QGuiApplication::sendEvent(&window, &pressEvent);
     QVERIFY(window.rootObject()->property("pressed").toBool());
     QVERIFY(!window.rootObject()->property("canceled").toBool());
-    QVERIFY(!window.rootObject()->property("released").toBool());
+    QCOMPARE(window.rootObject()->property("released").toInt(), expectedRelease);
+    QCOMPARE(window.rootObject()->property("clicked").toInt(), expectedClicks);
 
     QTest::qWait(200);
 
     //release
-    QMouseEvent releaseEvent(QEvent::MouseButtonRelease, QPoint(100, 100), Qt::LeftButton, Qt::LeftButton, 0);
     QGuiApplication::sendEvent(&window, &releaseEvent);
     QVERIFY(!window.rootObject()->property("pressed").toBool());
     QVERIFY(!window.rootObject()->property("canceled").toBool());
-    QVERIFY(window.rootObject()->property("released").toBool());
+    QCOMPARE(window.rootObject()->property("released").toInt(), ++expectedRelease);
+    QCOMPARE(window.rootObject()->property("clicked").toInt(), ++expectedClicks);
 }
 
 void tst_QQuickMouseArea::doubleClick()
@@ -941,17 +1032,17 @@ void tst_QQuickMouseArea::clickThrough()
     QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
+    // to avoid generating a double click.
+    const int doubleClickInterval = qApp->styleHints()->mouseDoubleClickInterval() + 10;
+
     QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
 
     QTRY_COMPARE(window->rootObject()->property("presses").toInt(), 0);
     QTRY_COMPARE(window->rootObject()->property("clicks").toInt(), 1);
 
-    // to avoid generating a double click.
-    int doubleClickInterval = qApp->styleHints()->mouseDoubleClickInterval() + 10;
-    QTest::qWait(doubleClickInterval);
-
-    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QCOMPARE(window->rootObject()->property("doubleClicks").toInt(), 0);
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100), doubleClickInterval);
     QTest::qWait(1000);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
 
@@ -981,9 +1072,7 @@ void tst_QQuickMouseArea::clickThrough()
     QCOMPARE(window->rootObject()->property("presses").toInt(), 0);
     QCOMPARE(window->rootObject()->property("clicks").toInt(), 0);
 
-    QTest::qWait(doubleClickInterval); // to avoid generating a double click.
-
-    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100), doubleClickInterval);
     QTest::qWait(1000);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
     QTest::qWait(100);
@@ -1002,15 +1091,13 @@ void tst_QQuickMouseArea::clickThrough()
 
     window->rootObject()->setProperty("letThrough", QVariant(true));
 
-    QTest::qWait(doubleClickInterval); // to avoid generating a double click.
-    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100), doubleClickInterval);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
 
     QCOMPARE(window->rootObject()->property("presses").toInt(), 0);
     QTRY_COMPARE(window->rootObject()->property("clicks").toInt(), 1);
 
-    QTest::qWait(doubleClickInterval); // to avoid generating a double click.
-    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100), doubleClickInterval);
     QTest::qWait(1000);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
     QTest::qWait(100);
@@ -1029,12 +1116,10 @@ void tst_QQuickMouseArea::clickThrough()
 
     window->rootObject()->setProperty("noPropagation", QVariant(true));
 
-    QTest::qWait(doubleClickInterval); // to avoid generating a double click.
-    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100), doubleClickInterval);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
 
-    QTest::qWait(doubleClickInterval); // to avoid generating a double click.
-    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100), doubleClickInterval);
     QTest::qWait(1000);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
     QTest::qWait(100);
@@ -1055,7 +1140,7 @@ void tst_QQuickMouseArea::clickThrough()
     QVERIFY(QTest::qWaitForWindowExposed(window.data()));
     QVERIFY(window->rootObject() != 0);
 
-    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100), doubleClickInterval);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
 
     QCOMPARE(window->rootObject()->property("clicksEnabled").toInt(), 1);
@@ -1063,12 +1148,24 @@ void tst_QQuickMouseArea::clickThrough()
 
     window->rootObject()->setProperty("disableLower", QVariant(true));
 
-    QTest::qWait(doubleClickInterval); // to avoid generating a double click.
-    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100), doubleClickInterval);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
 
     QCOMPARE(window->rootObject()->property("clicksEnabled").toInt(), 2);
     QCOMPARE(window->rootObject()->property("clicksDisabled").toInt(), 1); //disabled, shouldn't increment
+
+    window.reset(new QQuickView);
+
+    //QTBUG-49100
+    QVERIFY2(initView(*window.data(), testFileUrl("qtbug49100.qml"), true, &errorMessage), errorMessage.constData());
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
+    QVERIFY(window->rootObject() != 0);
+
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, QPoint(100,100));
+
+    QVERIFY(window->rootObject() != 0);
 }
 
 void tst_QQuickMouseArea::hoverPosition()
@@ -1188,7 +1285,7 @@ void tst_QQuickMouseArea::disableAfterPress()
     // target
     QQuickItem *blackRect = window.rootObject()->findChild<QQuickItem*>("blackrect");
     QVERIFY(blackRect != 0);
-    QVERIFY(blackRect == drag->target());
+    QCOMPARE(blackRect, drag->target());
 
     QVERIFY(!drag->active());
 
@@ -1481,7 +1578,7 @@ void tst_QQuickMouseArea::changeAxis()
     // target
     QQuickItem *blackRect = view.rootObject()->findChild<QQuickItem*>("blackrect");
     QVERIFY(blackRect != 0);
-    QVERIFY(blackRect == drag->target());
+    QCOMPARE(blackRect, drag->target());
 
     QVERIFY(!drag->active());
 
@@ -1639,6 +1736,69 @@ void tst_QQuickMouseArea::nestedStopAtBounds()
     QTest::mouseRelease(&view, Qt::LeftButton, 0, position);
 }
 
+void tst_QQuickMouseArea::containsPress_data()
+{
+    QTest::addColumn<bool>("hoverEnabled");
+
+    QTest::newRow("hover enabled") << true;
+    QTest::newRow("hover disaabled") << false;
+}
+
+void tst_QQuickMouseArea::containsPress()
+{
+    QFETCH(bool, hoverEnabled);
+
+    QQuickView window;
+    QByteArray errorMessage;
+    QVERIFY2(initView(window, testFileUrl("containsPress.qml"), true, &errorMessage), errorMessage.constData());
+    window.show();
+    window.requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+    QQuickItem *root = window.rootObject();
+    QVERIFY(root != 0);
+
+    QQuickMouseArea *mouseArea = window.rootObject()->findChild<QQuickMouseArea*>("mouseArea");
+    QVERIFY(mouseArea != 0);
+
+    QSignalSpy containsPressSpy(mouseArea, SIGNAL(containsPressChanged()));
+
+    mouseArea->setHoverEnabled(hoverEnabled);
+
+    QTest::mouseMove(&window, QPoint(22,33));
+    QCOMPARE(mouseArea->hovered(), false);
+    QCOMPARE(mouseArea->pressed(), false);
+    QCOMPARE(mouseArea->containsPress(), false);
+
+    QTest::mouseMove(&window, QPoint(200,200));
+    QCOMPARE(mouseArea->hovered(), hoverEnabled);
+    QCOMPARE(mouseArea->pressed(), false);
+    QCOMPARE(mouseArea->containsPress(), false);
+
+    QTest::mousePress(&window, Qt::LeftButton, Qt::NoModifier, QPoint(200,200));
+    QCOMPARE(mouseArea->hovered(), true);
+    QTRY_COMPARE(mouseArea->pressed(), true);
+    QCOMPARE(mouseArea->containsPress(), true);
+    QCOMPARE(containsPressSpy.count(), 1);
+
+    QTest::mouseMove(&window, QPoint(22,33));
+    QCOMPARE(mouseArea->hovered(), false);
+    QCOMPARE(mouseArea->pressed(), true);
+    QCOMPARE(mouseArea->containsPress(), false);
+    QCOMPARE(containsPressSpy.count(), 2);
+
+    QTest::mouseMove(&window, QPoint(200,200));
+    QCOMPARE(mouseArea->hovered(), true);
+    QCOMPARE(mouseArea->pressed(), true);
+    QCOMPARE(mouseArea->containsPress(), true);
+    QCOMPARE(containsPressSpy.count(), 3);
+
+    QTest::mouseRelease(&window, Qt::LeftButton, Qt::NoModifier, QPoint(200,200));
+    QCOMPARE(mouseArea->hovered(), hoverEnabled);
+    QCOMPARE(mouseArea->pressed(), false);
+    QCOMPARE(mouseArea->containsPress(), false);
+    QCOMPARE(containsPressSpy.count(), 4);
+}
+
 QTEST_MAIN(tst_QQuickMouseArea)
 
 #include "tst_qquickmousearea.moc"
