@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -161,6 +161,8 @@ private slots:
 
     void touchGrabCausesMouseUngrab();
 
+    void hoverEnabled();
+
 protected:
     bool eventFilter(QObject *, QEvent *event)
     {
@@ -589,10 +591,13 @@ void tst_TouchMouse::buttonOnFlickable()
     QPoint p2 = p1 + QPoint(0, -10);
     QPoint p3 = p2 + QPoint(0, -10);
     QQuickTouchUtils::flush(window);
+    QTest::qWait(1); // because Flickable pays attention to velocity, we need some time between movements
     QTest::touchEvent(window, device).move(0, p1, window);
     QQuickTouchUtils::flush(window);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p2, window);
     QQuickTouchUtils::flush(window);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p3, window);
     QQuickTouchUtils::flush(window);
 
@@ -674,10 +679,13 @@ void tst_TouchMouse::buttonOnDelayedPressFlickable()
     QPoint p2 = p1 + QPoint(0, -10);
     QPoint p3 = p2 + QPoint(0, -10);
     QQuickTouchUtils::flush(window);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p1, window);
     QQuickTouchUtils::flush(window);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p2, window);
     QQuickTouchUtils::flush(window);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p3, window);
     QQuickTouchUtils::flush(window);
     QVERIFY(flickable->isMovingVertically());
@@ -863,21 +871,25 @@ void tst_TouchMouse::pinchOnFlickable()
     QVERIFY(rect);
 
     // flickable - single touch point
-    QVERIFY(flickable->contentX() == 0.0);
+    QCOMPARE(flickable->contentX(), 0.0);
     QPoint p = QPoint(100, 100);
     QTest::touchEvent(window, device).press(0, p, window);
     QQuickTouchUtils::flush(window);
     QCOMPARE(rect->position(), QPointF(200.0, 200.0));
     p -= QPoint(10, 0);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p, window);
     QQuickTouchUtils::flush(window);
     p -= QPoint(10, 0);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p, window);
     QQuickTouchUtils::flush(window);
     p -= QPoint(10, 0);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p, window);
     QQuickTouchUtils::flush(window);
     p -= QPoint(10, 0);
+    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p, window);
     QQuickTouchUtils::flush(window);
     QTest::touchEvent(window, device).release(0, p, window);
@@ -945,7 +957,7 @@ void tst_TouchMouse::flickableOnPinch()
     QVERIFY(rect);
 
     // flickable - single touch point
-    QVERIFY(flickable->contentX() == 0.0);
+    QCOMPARE(flickable->contentX(), 0.0);
     QPoint p = QPoint(100, 100);
     QTest::touchEvent(window, device).press(0, p, window);
     QQuickTouchUtils::flush(window);
@@ -1025,7 +1037,7 @@ void tst_TouchMouse::mouseOnFlickableOnPinch()
     QVERIFY(rect);
 
     // flickable - single touch point
-    QVERIFY(flickable->contentX() == 0.0);
+    QCOMPARE(flickable->contentX(), 0.0);
     QPoint p = QPoint(100, 100);
     QTest::touchEvent(window, device).press(0, p, window);
     QQuickTouchUtils::flush(window);
@@ -1090,13 +1102,16 @@ void tst_TouchMouse::mouseOnFlickableOnPinch()
     QQuickTouchUtils::flush(window);
     QCOMPARE(rect->position(), QPointF(200.0, 200.0));
     p -= QPoint(10, 0);
+    QTest::qWait(1);
     pinchSequence.move(0, p, window).commit();
     QQuickTouchUtils::flush(window);
     p -= QPoint(10, 0);
+    QTest::qWait(1);
     pinchSequence.move(0, p, window).commit();
     QQuickTouchUtils::flush(window);
     QGuiApplication::processEvents();
     p -= QPoint(10, 0);
+    QTest::qWait(1);
     pinchSequence.move(0, p, window).commit();
     QQuickTouchUtils::flush(window);
 
@@ -1232,6 +1247,93 @@ void tst_TouchMouse::touchGrabCausesMouseUngrab()
     delete window;
 }
 
+void tst_TouchMouse::hoverEnabled()
+{
+    // QTouchDevice *device = new QTouchDevice;
+    // device->setType(QTouchDevice::TouchScreen);
+    // QWindowSystemInterface::registerTouchDevice(device);
+
+    QQuickView *window = createView();
+    window->setSource(testFileUrl("hoverMouseAreas.qml"));
+
+    window->show();
+    window->requestActivate();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
+    QQuickItem *root = window->rootObject();
+    QVERIFY(root != 0);
+
+    QQuickMouseArea *mouseArea1 = root->findChild<QQuickMouseArea*>("mouseArea1");
+    QVERIFY(mouseArea1 != 0);
+
+    QQuickMouseArea *mouseArea2 = root->findChild<QQuickMouseArea*>("mouseArea2");
+    QVERIFY(mouseArea2 != 0);
+
+    QSignalSpy enterSpy1(mouseArea1, SIGNAL(entered()));
+    QSignalSpy exitSpy1(mouseArea1, SIGNAL(exited()));
+    QSignalSpy clickSpy1(mouseArea1, SIGNAL(clicked(QQuickMouseEvent *)));
+
+    QSignalSpy enterSpy2(mouseArea2, SIGNAL(entered()));
+    QSignalSpy exitSpy2(mouseArea2, SIGNAL(exited()));
+    QSignalSpy clickSpy2(mouseArea2, SIGNAL(clicked(QQuickMouseEvent *)));
+
+    QPoint p0(50, 50);
+    QPoint p1(150, 150);
+    QPoint p2(150, 250);
+
+    // ------------------------- Mouse move to mouseArea1
+    QTest::mouseMove(window, p1);
+
+    QVERIFY(enterSpy1.count() == 1);
+    QVERIFY(mouseArea1->hovered());
+    QVERIFY(!mouseArea2->hovered());
+
+    // ------------------------- Touch click on mouseArea1
+    QTest::touchEvent(window, device).press(0, p1, window);
+
+    QVERIFY(enterSpy1.count() == 1);
+    QVERIFY(enterSpy2.count() == 0);
+    QVERIFY(mouseArea1->pressed());
+    QVERIFY(mouseArea1->hovered());
+    QVERIFY(!mouseArea2->hovered());
+
+    QTest::touchEvent(window, device).release(0, p1, window);
+    QVERIFY(clickSpy1.count() == 1);
+    QVERIFY(mouseArea1->hovered());
+    QVERIFY(!mouseArea2->hovered());
+
+    // ------------------------- Touch click on mouseArea2
+    QTest::touchEvent(window, device).press(0, p2, window);
+
+    QVERIFY(mouseArea1->hovered());
+    QVERIFY(mouseArea2->hovered());
+    QVERIFY(mouseArea2->pressed());
+    QVERIFY(enterSpy1.count() == 1);
+    QVERIFY(enterSpy2.count() == 1);
+
+    QTest::touchEvent(window, device).release(0, p2, window);
+
+    QVERIFY(clickSpy2.count() == 1);
+    QVERIFY(mouseArea1->hovered());
+    QVERIFY(!mouseArea2->hovered());
+    QVERIFY(exitSpy1.count() == 0);
+    QVERIFY(exitSpy2.count() == 1);
+
+    // ------------------------- Another touch click on mouseArea1
+    QTest::touchEvent(window, device).press(0, p1, window);
+
+    QVERIFY(enterSpy1.count() == 1);
+    QVERIFY(enterSpy2.count() == 1);
+    QVERIFY(mouseArea1->pressed());
+    QVERIFY(mouseArea1->hovered());
+    QVERIFY(!mouseArea2->hovered());
+
+    QTest::touchEvent(window, device).release(0, p1, window);
+    QVERIFY(clickSpy1.count() == 2);
+    QVERIFY(mouseArea1->hovered());
+    QVERIFY(!mouseArea1->pressed());
+    QVERIFY(!mouseArea2->hovered());
+}
+
 QTEST_MAIN(tst_TouchMouse)
 
 #include "tst_touchmouse.moc"
