@@ -1,40 +1,32 @@
 /****************************************************************************
 **
 ** Copyright (C) 2013 Research In Motion.
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -47,6 +39,8 @@
 #include <QGuiApplication>
 #include <QWindow>
 #include <QFileOpenEvent>
+#include <QOpenGLContext>
+#include <QOpenGLFunctions>
 #ifdef QT_WIDGETS_LIB
 #include <QApplication>
 #endif // QT_WIDGETS_LIB
@@ -59,10 +53,12 @@
 #include <QFileInfo>
 #include <QRegularExpression>
 #include <QStringList>
+#include <QScopedPointer>
 #include <QDebug>
 #include <QStandardPaths>
 #include <QTranslator>
 #include <QtGlobal>
+#include <QLibraryInfo>
 #include <qqml.h>
 #include <qqmldebug.h>
 #include <private/qabstractanimation_p.h>
@@ -72,14 +68,15 @@
 #include <cstdlib>
 
 #define VERSION_MAJ 1
-#define VERSION_MIN 0
-#define VERSION_STR "1.0"
+#define VERSION_MIN 1
+#define VERSION_STR "1.1"
 
 #define FILE_OPEN_EVENT_WAIT_TIME 3000 // ms
 
 static Config *conf = 0;
 static QQmlApplicationEngine *qae = 0;
 static int exitTimerId = -1;
+bool verboseMode = false;
 
 static void loadConf(const QString &override, bool quiet) // Terminates app on failure
 {
@@ -108,6 +105,7 @@ static void loadConf(const QString &override, bool quiet) // Terminates app on f
     }
 
     if (!quiet) {
+        printf("qml: %s\n", QLibraryInfo::build());
         if (builtIn)
             printf("qml: Using built-in configuration.\n");
         else
@@ -128,20 +126,6 @@ static void loadConf(const QString &override, bool quiet) // Terminates app on f
     }
 }
 
-void contain(QObject *o, const QUrl &containPath)
-{
-    QQmlComponent c(qae, containPath);
-    QObject *o2 = c.create();
-    if (!o2)
-        return;
-    bool success = false;
-    int idx;
-    if ((idx = o2->metaObject()->indexOfProperty("containedObject")) != -1)
-        success = o2->metaObject()->property(idx).write(o2, QVariant::fromValue<QObject*>(o));
-    if (!success)
-        o->setParent(o2); //Set QObject parent, and assume container will react as needed
-}
-
 #ifdef QT_GUI_LIB
 
 void noFilesGiven();
@@ -180,14 +164,24 @@ class LoadWatcher : public QObject
 public:
     LoadWatcher(QQmlApplicationEngine *e, int expected)
         : QObject(e)
+        , earlyExit(false)
         , expect(expected)
         , haveOne(false)
     {
         connect(e, SIGNAL(objectCreated(QObject*,QUrl)),
             this, SLOT(checkFinished(QObject*)));
+        // QQmlApplicationEngine also connects quit() to QCoreApplication::quit
+        // but if called before exec() then QCoreApplication::quit does nothing
+        connect(e, SIGNAL(quit()),
+            this, SLOT(quit()));
     }
 
+    bool earlyExit;
+
 private:
+    void contain(QObject *o, const QUrl &containPath);
+    void checkForWindow(QObject *o);
+
     int expect;
     bool haveOne;
 
@@ -195,6 +189,7 @@ public Q_SLOTS:
     void checkFinished(QObject *o)
     {
         if (o) {
+            checkForWindow(o);
             haveOne = true;
             if (conf && qae)
                 foreach (PartialScene *ps, conf->completers)
@@ -209,8 +204,61 @@ public Q_SLOTS:
             exit(2);//Different return code from qFatal
         }
     }
+
+    void quit() {
+        //Will be checked before calling exec()
+        earlyExit = true;
+    }
+#if defined(QT_GUI_LIB) && !defined(QT_NO_OPENGL)
+    void onOpenGlContextCreated(QOpenGLContext *context);
+#endif
 };
 
+void LoadWatcher::contain(QObject *o, const QUrl &containPath)
+{
+    QQmlComponent c(qae, containPath);
+    QObject *o2 = c.create();
+    if (!o2)
+        return;
+    checkForWindow(o2);
+    bool success = false;
+    int idx;
+    if ((idx = o2->metaObject()->indexOfProperty("containedObject")) != -1)
+        success = o2->metaObject()->property(idx).write(o2, QVariant::fromValue<QObject*>(o));
+    if (!success)
+        o->setParent(o2); //Set QObject parent, and assume container will react as needed
+}
+
+void LoadWatcher::checkForWindow(QObject *o)
+{
+#if defined(QT_GUI_LIB) && !defined(QT_NO_OPENGL)
+    if (verboseMode && o->isWindowType() && o->inherits("QQuickWindow")) {
+        connect(o, SIGNAL(openglContextCreated(QOpenGLContext*)),
+                this, SLOT(onOpenGlContextCreated(QOpenGLContext*)));
+    }
+#else
+    Q_UNUSED(o)
+#endif // QT_GUI_LIB && !QT_NO_OPENGL
+}
+
+#if defined(QT_GUI_LIB) && !defined(QT_NO_OPENGL)
+void LoadWatcher::onOpenGlContextCreated(QOpenGLContext *context)
+{
+    context->makeCurrent(qobject_cast<QWindow *>(sender()));
+    QOpenGLFunctions functions(context);
+    QByteArray output = "Vendor  : ";
+    output += reinterpret_cast<const char *>(functions.glGetString(GL_VENDOR));
+    output += "\nRenderer: ";
+    output += reinterpret_cast<const char *>(functions.glGetString(GL_RENDERER));
+    output += "\nVersion : ";
+    output += reinterpret_cast<const char *>(functions.glGetString(GL_VERSION));
+    output += "\nLanguage: ";
+    output += reinterpret_cast<const char *>(functions.glGetString(GL_SHADING_LANGUAGE_VERSION));
+    puts(output.constData());
+    context->doneCurrent();
+}
+#endif // QT_GUI_LIB && !QT_NO_OPENGL
+
 void quietMessageHandler(QtMsgType type, const QMessageLogContext &ctxt, const QString &msg)
 {
     Q_UNUSED(ctxt);
@@ -246,7 +294,6 @@ QmlApplicationType applicationType = QmlApplicationTypeCore;
 QmlApplicationType applicationType = QmlApplicationTypeGui;
 #endif // QT_GUI_LIB
 bool quietMode = false;
-bool verboseMode = false;
 void printVersion()
 {
     printf("qml binary version ");
@@ -259,11 +306,10 @@ void printVersion()
 
 void printUsage()
 {
-    printf("Usage: qml [options] [files]\n");
+    printf("Usage: qml [options] [files] [-- args]\n");
     printf("\n");
-    printf("Any argument ending in .qml will be treated as a QML file to be loaded.\n");
+    printf("Any unknown argument before '--' will be treated as a QML file to be loaded.\n");
     printf("Any number of QML files can be loaded. They will share the same engine.\n");
-    printf("Any argument which is not a recognized option and which does not end in .qml will be ignored.\n");
     printf("'gui' application type is only available if the QtGui module is available.\n");
     printf("'widget' application type is only available if the QtWidgets module is available.\n");
     printf("\n");
@@ -282,6 +328,12 @@ void printUsage()
     printf("\t-f [file] .................... Load the given file as a QML file.\n");
     printf("\t-config [file] ............... Load the given file as the configuration file.\n");
     printf("\t-- ........................... Arguments after this one are ignored by the launcher, but may be used within the QML application.\n");
+    printf("\tGL options:\n");
+    printf("\t-desktop.......................Force use of desktop GL (AA_UseDesktopOpenGL)\n");
+    printf("\t-gles..........................Force use of GLES (AA_UseOpenGLES)\n");
+    printf("\t-software......................Force use of software rendering (AA_UseOpenGLES)\n");
+    printf("\t-scaling.......................Enable High DPI scaling (AA_EnableHighDpiScaling)\n");
+    printf("\t-no-scaling....................Disable High DPI scaling (AA_DisableHighDpiScaling)\n");
     printf("\tDebugging options:\n");
     printf("\t-verbose ..................... Print information about what qml is doing, like specific file urls being loaded.\n");
     printf("\t-translation [file] .......... Load the given file as the translations file.\n");
@@ -393,11 +445,11 @@ int main(int argc, char *argv[])
     }
 
     app->setApplicationName("Qml Runtime");
-    app->setOrganizationName("Qt Project");
+    app->setOrganizationName("QtProject");
     app->setOrganizationDomain("qt-project.org");
 
-    qmlRegisterType<Config>("QmlRuntime.Config", VERSION_MAJ, VERSION_MIN, "Configuration");
-    qmlRegisterType<PartialScene>("QmlRuntime.Config", VERSION_MAJ, VERSION_MIN, "PartialScene");
+    qmlRegisterType<Config>("QmlRuntime.Config", 1, 0, "Configuration");
+    qmlRegisterType<PartialScene>("QmlRuntime.Config", 1, 0, "PartialScene");
     QQmlApplicationEngine e;
     QStringList files;
     QString confFile;
@@ -406,7 +458,7 @@ int main(int argc, char *argv[])
 
     //Handle main arguments
     QStringList argList = app->arguments();
-    for (int i = 0; i < argList.count(); i++) {
+    for (int i = 1; i < argList.count(); i++) {
         const QString &arg = argList[i];
         if (arg == QLatin1String("-quiet"))
             quietMode = true;
@@ -447,10 +499,18 @@ int main(int argc, char *argv[])
                 continue;//Invalid usage, but just ignore it
             dummyDir = argList[i+1];
             i++;
+        } else if (arg == QLatin1String("-gles")) {
+            QCoreApplication::setAttribute(Qt::AA_UseOpenGLES);
+        } else if (arg == QLatin1String("-software")) {
+            QCoreApplication::setAttribute(Qt::AA_UseSoftwareOpenGL);
+        } else if (arg == QLatin1String("-desktop")) {
+            QCoreApplication::setAttribute(Qt::AA_UseDesktopOpenGL);
+        } else if (arg == QLatin1String("-scaling")) {
+            QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+        } else if (arg == QLatin1String("-no-scaling")) {
+            QCoreApplication::setAttribute(Qt::AA_DisableHighDpiScaling);
         } else {
-            //If it ends in .qml, treat it as a file. Else ignore it
-            if (arg.endsWith(".qml"))
-                files << arg;
+            files << arg;
         }
     }
 
@@ -494,7 +554,7 @@ int main(int argc, char *argv[])
     loadConf(confFile, !verboseMode);
 
     //Load files
-    LoadWatcher lw(&e, files.count());
+    QScopedPointer<LoadWatcher> lw(new LoadWatcher(&e, files.count()));
 
     // Load dummy data before loading QML-files
     if (!dummyDir.isEmpty() && QFileInfo (dummyDir).isDir())
@@ -502,6 +562,7 @@ int main(int argc, char *argv[])
 
     foreach (const QString &path, files) {
         //QUrl::fromUserInput doesn't treat no scheme as relative file paths
+#ifndef QT_NO_REGULAREXPRESSION
         QRegularExpression urlRe("[[:word:]]+://.*");
         if (urlRe.match(path).hasMatch()) { //Treat as a URL
             QUrl url = QUrl::fromUserInput(path);
@@ -511,7 +572,9 @@ int main(int argc, char *argv[])
                         ? QDir::toNativeSeparators(url.toLocalFile())
                         : url.toString()));
             e.load(url);
-        } else { //Local file path
+        } else
+#endif
+        { //Local file path
             if (verboseMode)
                 printf("qml: loading %s\n", qPrintable(QDir::toNativeSeparators(path)));
 
@@ -523,6 +586,9 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (lw->earlyExit)
+        return 0;
+
     return app->exec();
 }
 
